#!/bin/bash

# This script runs as https://build.eclipse.org/hudson/job/cbi-*/configure
# and is archived in  http://dev.eclipse.org/viewcvs/index.cgi/org.eclipse.dash/athena/org.eclipse.dash.commonbuilder/org.eclipse.dash.commonbuilder.releng/hudson/?root=Technology_Project
# Build runs under ${WORKSPACE} == /opt/users/hudsonbuild/.hudson/jobs/cbi-*/workspace

echo "[`date +%Y/%m/%d\ %H:%M`] Hudson job ${JOBNAME} build #${BUILD_NUMBER} (${BUILD_ID}) started." 

##############################################################################################

# BEGIN CONFIGURATION

buildTimestamp="`date +%Y%m%d%H%M`"
projectid="technology.linuxtools"
version="0.2.0"
javaHome="/opt/public/common/ibm-java-jdk-ppc-60"
basebuilderBranch="R35_M5"
if [[ $BUILDTYPE ]]; then 
	buildType="$BUILDTYPE"; # exposed as a Hudson build parameter for convenience
else
	buildType="N"; # skip signing for type N 
fi

# pass in additional flags like -buildAlias=1.0.0RC2 using the $EXTRAFLAGS Hudson parameter
# buildAlias will rename zips from foo-SDK-N200901011234.zip to foo-SDK-1.0.0RC2.zip

##############################################################################################

# define where to do all the work; start with a fresh folder each time
writableBuildRoot="${WORKSPACE}/build"
if [[ -d ${writableBuildRoot} ]]; then rm -fr ${writableBuildRoot}; fi

# define required folders
downloadsDir="${writableBuildRoot}/downloads"
signingDir="${writableBuildRoot}/signing"

# long form (default if omitted)
# buildDir="${writableBuildRoot}/${projectid//.//}/downloads/drops/${version}/${buildType}${buildTimestamp}"
# short form (non-default)
buildDir="${writableBuildRoot}/${buildType}${buildTimestamp}"

# create required folders & files (as symlinks is possible)
mkdir -p ${downloadsDir} ${signingDir} ${buildDir}

# create .cvspass file to shut up unnecessary warnings
touch ${writableBuildRoot}/.cvspass

#define symlinked required folders
relengBaseBuilderDir="${writableBuildRoot}/org.eclipse.releng.basebuilder"
relengCommonBuilderDir="${writableBuildRoot}/org.eclipse.dash.common.releng"
# symlink basebuilder and common.releng; alternatively, if you omit this, they'll be checked out in start.sh
ln -s /opt/public/cbi/build/org.eclipse.releng.basebuilder ${writableBuildRoot}/
ln -s /opt/public/cbi/build/org.eclipse.dash.common.releng ${writableBuildRoot}/

# symlink 3rdPartyJars (reuse existing content on build.eclipse.org for ant-contrib.jar, etc.)
thirdPartyJarsDir="${writableBuildRoot}/3rdPartyJars"
ln -s /opt/public/cbi/build/3rdPartyJars ${writableBuildRoot}/

# run a build
cd ${writableBuildRoot}/org.eclipse.dash.common.releng/tools/scripts
./start.sh -projectid ${projectid} -version ${version} \
  -buildType ${buildType} -buildTimestamp ${buildTimestamp} \
  -writableBuildRoot ${writableBuildRoot} \
  -thirdPartyJarsDir ${thirdPartyJarsDir} -downloadsDir ${downloadsDir} -buildDir ${buildDir} \
  -basebuilderBranch ${basebuilderBranch} -javaHome ${javaHome} \
  -URL http://download.eclipse.org/eclipse/downloads/drops/R-3.4.1-200809111700/eclipse-SDK-3.4.1-linux-gtk-ppc.tar.gz \
  -URL http://download.eclipse.org/tools/cdt/releases/ganymede/dist/cdt-master-5.0.0.zip \
  -URL http://download.eclipse.org/birt/downloads/drops/R-R1-2_3_1-200809221151/birt-report-framework-2_3_1.zip \
  -URL http://download.eclipse.org/modeling/emf/emf/downloads/drops/2.4.1/R200808251517/emf-runtime-2.4.1.zip \
  -projRelengRoot ':pserver:anonymous@dev.eclipse.org:/cvsroot/technology' \
  -projRelengPath 'org.eclipse.dash/athena/org.eclipse.dash.commonbuilder/org.eclipse.linuxtools.releng' \
  ${EXTRAFLAGS} 2>&1
  
  # remove file so workspace navigation is one click simpler
rm -f ${writableBuildRoot}/.cvspass

echo "[`date +%Y/%m/%d\ %H:%M`] Hudson job ${JOBNAME} build #${BUILD_NUMBER} (${BUILD_ID}) done." 
  
# SVN build 
#  -projRelengRoot 'http://dev.eclipse.org/svnroot/technology/org.eclipse.linuxtools/releng/trunk' \
#  -projRelengPath 'org.eclipse.linuxtools.releng' \


# TODO: [253269] run an Ant build instead of a shell script which wraps an Ant build