#!/bin/bash

# This script runs as https://build.eclipse.org/hudson/job/cbi-*/configure
# and is archived in  http://dev.eclipse.org/viewcvs/index.cgi/org.eclipse.dash/athena/org.eclipse.dash.commonbuilder/org.eclipse.dash.commonbuilder.releng/hudson/?root=Technology_Project
# Build runs under ${WORKSPACE} == /opt/users/hudsonbuild/.hudson/jobs/cbi-*/workspace

##############################################################################################

# 1. configure build here, and below if required
buildTimestamp="`date +%Y%m%d%H%M`"
projectid="technology.linuxtools"
version="0.2.0"
javaHome="/opt/public/common/ibm-java-jdk-ppc-60"
basebuilderBranch="R35_M4"
if [[ $BUILDTYPE ]]; then 
	buildType="$BUILDTYPE"; # exposed as a Hudson build parameter for convenience
else
	buildType="N"; # skip signing for type N 
fi

##############################################################################################

# define required folders
downloadsDir="${WORKSPACE}/build/downloads"
thirdPartyJarsDir="${WORKSPACE}/build/3rdPartyJars"
if [[ ${projectid%.*} == ${projectid%%.*} ]]; then # two-part projectid; single-match trim .* and greedy-match trim .* are the same
	topprojectName=${projectid%%.*}; # get first chunk
	projectName=${projectid##*.}; # get last chunk
	subprojectName=${projectName}; # proj == sub
else # assume x.y.z -> top.proj.sub 
	topprojectName=${projectid%%.*}; # get first chunk
	subprojectName=${projectid##*.}; # get last chunk
	projectName=${projectid#${topprojectName}.}; # trim first chunk
	projectName=${projectName%.${subprojectName}}; # trim last chunk
fi				
if [[ $projectName == $subprojectName ]]; then
	buildDir="${WORKSPACE}/build/${projectName}/downloads/drops/${version}/${buildType}${buildTimestamp}"
else
	buildDir="${WORKSPACE}/build/${projectName}/${subprojectName}/downloads/drops/${version}/${buildType}${buildTimestamp}"
fi

# create required folders & files (as symlinks is possible)
if [[ -d ${WORKSPACE}/build ]]; then rm -fr ${WORKSPACE}/build; fi
mkdir -p ${downloadsDir} ${buildDir}
touch ${WORKSPACE}/.cvspass

# symlink the common builder and base builder
if [[ -L ${buildDir}/org.eclipse.releng.basebuilder ]]; then rm -f ${buildDir}/org.eclipse.releng.basebuilder; fi
if [[ -L ${buildDir}/org.eclipse.dash.common.releng ]]; then rm -f ${buildDir}/org.eclipse.dash.common.releng; fi
ln -s /opt/public/cbi/build/org.eclipse.releng.basebuilder ${buildDir}/
ln -s /opt/public/cbi/build/org.eclipse.dash.common.releng ${buildDir}/

# TODO: is this required? symlink the 3rd party jars dir
if [[ -L ${thirdPartyJarsDir} ]]; then rm -f ${thirdPartyJarsDir}; fi
ln -s /opt/public/cbi/build/3rdPartyJars ${WORKSPACE}/build/

# TODO: shouldn't need to duplicate the common builder, but start.sh still needs it
if [[ -L ${WORKSPACE}/build/org.eclipse.dash.common.releng ]]; then rm -f ${WORKSPACE}/build/org.eclipse.dash.common.releng; fi
ln -s /opt/public/cbi/build/org.eclipse.dash.common.releng ${WORKSPACE}/build/

# 2. run a build, using specified .releng project and input dependency URL(s)
cd ${WORKSPACE}/build/org.eclipse.dash.common.releng/tools/scripts
./start.sh -projectid ${projectid} -version ${version} -writableBuildRoot ${WORKSPACE} \
  -buildType ${buildType} -buildTimestamp ${buildTimestamp} \
  -basebuilderBranch ${basebuilderBranch} -javaHome ${javaHome} -downloadsDir ${downloadsDir} \
  -URL http://download.eclipse.org/eclipse/downloads/drops/R-3.4.1-200809111700/eclipse-SDK-3.4.1-linux-gtk-ppc.tar.gz \
  -URL http://download.eclipse.org/tools/cdt/releases/ganymede/dist/cdt-master-5.0.0.zip \
  -URL http://download.eclipse.org/birt/downloads/drops/R-R1-2_3_1-200809221151/birt-report-framework-2_3_1.zip \
  -URL http://download.eclipse.org/modeling/emf/emf/downloads/drops/2.4.1/R200808251517/emf-runtime-2.4.1.zip \
  -projRelengRoot ':pserver:anonymous@dev.eclipse.org:/cvsroot/technology' \
  -projRelengPath 'org.eclipse.dash/athena/org.eclipse.dash.commonbuilder/org.eclipse.linuxtools.releng'
  
# SVN build 
#  -projRelengRoot 'http://dev.eclipse.org/svnroot/technology/org.eclipse.linuxtools/releng/trunk' \
#  -projRelengPath 'org.eclipse.linuxtools.releng' \


# TODO: [253269] run an Ant build instead of a shell script which wraps an Ant build