/*
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   IBM - initial API and implementation
 */

package org.eclipse.dash.common.build.taskdefs;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.Vector;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;


public class CreateTestManifestFile extends Task
{
  private static final String TEST_MANIFEST_CREATED_SUCCESSFULLY = "Test manifest created successfully."; //$NON-NLS-1$

  private static final String ENTRIES = "@entries@"; //$NON-NLS-1$

  private static final String NL = "\n"; //$NON-NLS-1$

  private static final String LOG_FILE = "</logFile>\n"; //$NON-NLS-1$

  private static final String EFFECTED_FILE = "\"></effectedFile>\n"; //$NON-NLS-1$

  private static final String EFFECTED_FILE_ID = "\t<effectedFile id=\""; //$NON-NLS-1$

  private static final String BIN_LOG = "_*/*.bin.log\">\n"; //$NON-NLS-1$

  private static final String LOG_FILE_NAME_PLUGINS = "<logFile name=\"plugins/"; //$NON-NLS-1$

  private static final String PROJ_RUN = "projRUN"; //$NON-NLS-1$

  private static final String SDK = "SDK"; //$NON-NLS-1$

  private static final String EX = "EX"; //$NON-NLS-1$

  private static final String T = "T"; //$NON-NLS-1$

  private static final String PLUGINS = "plugins"; //$NON-NLS-1$

  private static final String EXAMPLES = "examples"; //$NON-NLS-1$

  private static final String TESTS = "tests"; //$NON-NLS-1$

  private static final String FEATURE = "-feature$"; //$NON-NLS-1$

  private static final String FRAGMENT = "-fragment$"; //$NON-NLS-1$

  private static final String REGEX_DIR = "(\\S+) ?"; //$NON-NLS-1$

  private static final String EMPTY = ""; //$NON-NLS-1$

  private Vector directories;

  private String templatefile;

  private String targetfile;

  public CreateTestManifestFile()
  {
    directories = new Vector();
    templatefile = EMPTY;
    targetfile = EMPTY;
  }

  public void setDirectories(String d)
  {
    Pattern p = Pattern.compile(REGEX_DIR);
    Matcher m = p.matcher(d);
    while (m.find())
    {
      directories.add(new String(m.group(1)));
      m = p.matcher(m.replaceFirst(EMPTY));
    }
  }

  public void setTemplatefile(String t)
  {
    templatefile = t;
  }

  public void setTargetfile(String t)
  {
    targetfile = t;
  }

  public void execute() throws BuildException
  {
    String[] entries = new String [directories.size()];
    directories.toArray(entries);

    for (int i = 0; i < entries.length; i++)
    {
      entries[i] = Pattern.compile(FRAGMENT).matcher(entries[i]).replaceFirst(EMPTY);

      if (!Pattern.compile(FEATURE).matcher(entries[i]).find())
      {
        String elementid = basename(entries[i]);
        String[] type = { EMPTY, EMPTY };

        if (Pattern.compile(TESTS).matcher(entries[i]).find())
        {
          type[0] = T;
        }
        else if (Pattern.compile(EXAMPLES).matcher(entries[i]).find())
        {
          type[0] = EX;
        }
        else if (Pattern.compile(PLUGINS).matcher(entries[i]).find())
        {
          type[0] = SDK;
          type[1] = PROJ_RUN;
        }
        else
        {
          entries[i] = EMPTY;
          continue;
        }

        String entry = LOG_FILE_NAME_PLUGINS + elementid + BIN_LOG;
        for (int j = 0; j <= 1; j++)
        {
          if (!type[j].equals(EMPTY))
          {
            entry += EFFECTED_FILE_ID + type[j] + EFFECTED_FILE;
          }
        }
        entry += LOG_FILE;
        entries[i] = entry;
      }
      else
      {
        entries[i] = EMPTY;
      }
    }

    String allentries = join(NL, entries);
    String testtemplate = slurp(templatefile);

    testtemplate = Pattern.compile(ENTRIES, Pattern.MULTILINE).matcher(testtemplate).replaceAll(allentries);

    dump(targetfile, testtemplate);

    System.out.println(TEST_MANIFEST_CREATED_SUCCESSFULLY);
  }

  public String basename(String file)
  {
    File f = new File(file);

    return f.getName();
  }

  private String slurp(String file)
  {
    FileInputStream fin;
    try
    {
      fin = new FileInputStream(file);
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }
    int c;
    String str = EMPTY;

    try
    {
      while ((c = fin.read()) != -1)
      {
        str += (char)c;
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }

    try
    {
      fin.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }

    return str;
  }

  private String join(String glue, String[] arr)
  {
    String str = EMPTY;
    for (int i = 0; i < arr.length; i++)
    {
      if (!arr[i].equals(EMPTY))
      {
        str += arr[i] + glue;
      }
    }

    return str;
  }

  private void dump(String file, String str)
  {
    FileOutputStream fout;
    try
    {
      fout = new FileOutputStream(file);
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }
    byte[] b = new byte [str.length()];

    for (int i = 0; i < str.length(); i++)
    {
      b[i] = (byte)str.charAt(i);
    }

    try
    {
      fout.write(b);
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }

    try
    {
      fout.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }
  }
}
