#!/bin/bash

# this script is used by components' buildAll.xml scripts (via buildAllHelper.xml) to add any unjarred features or plugins (eg., source or doc)
# to its "master feature zip" so that they will be signed and packed for use on update sites
 
if [[ $# -eq 0 ]]; then
	echo "Usage: $0 /path/to/zipFileToUpdate.zip"; # add jarred features/plugins to existing zip
	echo "  - or -";
	echo "Usage: $0 /path/to/zipFileSource.zip /path/to/zipFileTarget.zip"; # add jarred features/plugins to a second zip
	exit 1;
fi

zip=$1;
if [[ $2 ]]; then destZip=$2; else destZip=$1; fi

echo "Source: $zip"
echo "Target: $destZip";

tmpdir=`mktemp -d`;

# unzip features & plugins into temp dir, then for each one, zip it as a jar and delete the orig folder
unzip -q $zip "eclipse/*" -d $tmpdir; 
for d in $tmpdir/eclipse/features/ $tmpdir/eclipse/plugins/; do
	pushd $d >/dev/null;
	#echo "In $d:"
	# remove any existing jars -- we only want folders 
	for j in $(find . -maxdepth 1 -type f -name "*.jar"); do
		rm -f $j;
	done
	for p in "org.*" "net.*"; do # echo "p=$p" 
		cnt=0;
		for f in $(find . -maxdepth 1 -type d -name "$p"); do # echo "f=$f";
			#echo "  Jar $f to ${f}.jar"
			(( cnt++ )); 
			pushd $f >/dev/null; 
			zip -ruq ${f}.jar ./* ./.??*; # include .dotfiles, but exclude parent dir ".."
			mv ${f}.jar ../;
			touch ../${f}.jar;
			popd >/dev/null; 
			rm -fr $f; 
		done
		echo "Jarred $cnt $p folders in $d";
	done
	popd >/dev/null;
done

# add the new jars to the zip, then cleanup
pushd $tmpdir >/dev/null;
zip -ruq $destZip eclipse/*
touch $destZip
popd >/dev/null;
rm -fr $tmpdir;

echo "Updated: $destZip";
