#!/bin/sh

function listDirs ()
{
    echo "";
    echo "Choose one of these folders to clean:";
    find . -maxdepth 2  -type d -regex ".*/[NMISR]20[0-1][0-9]+" -exec du -sh {} \; 2>/dev/null
    exit 1;
}

function usage ()
{
  echo "This script is used to purge temporary files leftover from a 'noclean' build.";
  echo "Must be run from parent folder of the build you wish to clean. Use -y to suppress confirmations.";
  echo "";
  echo "Usage:   $0 <version>/<buildID>";
  echo "Example: $0 -y 2.4.0/N200801101234 2.4.0/N200802011243 ...";
  listDirs;
}

cleanAll=0
dirs="";
while [ "$#" -gt 0 ]; do
	case $1 in
		'-y') cleanAll=1; shift 1;;
		*) dirs=$dirs" "$1; shift 1;;
	esac
done

if [[ ! $dirs ]] || [[ $dirs == "" ]]; then
	usage;
fi

for d in $dirs; do
	dir="";
	if [[ -d `pwd`/$d ]]; then
	  dir=`pwd`/$d;
	elif [[ -d $d ]]; then
	  dir=$d;
	fi
	if [[ $dir != "" ]]; then
	  if [[ $cleanAll -eq 1 ]]; then
	    echo "Cleaning $dir ..."; yn="y";
	  else
	    echo -n "OK to clean $dir? [y/N]"; read yn;
	  fi
	  if [[ $yn == "y" ]] || [[ $yn == "Y" ]]; then
	    pushd $dir 1>/dev/null;
	    echo "Size before: "$(du -sh . | perl -pe "s/\t\.//g");
	    if [[ -d eclipse/$dir ]]; then
	      mv eclipse/$dir/* .;
	    fi
	    if [[ -d eclipse ]]; then rm -fr eclipse; fi
	    if [[ -d testing ]]; then rm -fr testing; fi
	    rm -fr org.eclipse.* noclean;
	    echo "Size after:  "$(du -sh . | perl -pe "s/\t\.//g");
	    popd  1>/dev/null;
	  fi
	else
	  	echo "$1 is not a valid build folder (beneath the current directory).";
  		listDirs;
	fi
	shift 1;
done
