#!/bin/bash

norm="\033[0;39m";
grey="\033[1;30m";
green="\033[1;32m";
brown="\033[0;33m";
yellow="\033[1;33m";
blue="\033[1;34m";
cyan="\033[1;36m";
red="\033[1;31m";

function usage()
{
	echo "usage: $0"
	echo -e "${red}-a$norm, ${red}-active$norm   <required: used to prevent accidentally running this script w/o options>"
	echo "-debug        <optional: use -debug 1 or -debug 2 for more output>"
	echo "-p, -preview  <optional: don't actually run a build, just echo the command for preview>"
	echo ""
	exit 1
}

if [ $# -lt 1 ]; then
	usage;
fi

prefix="/home/data/httpd/download.eclipse.org"
searchDirs=( 
"$prefix/modeling/emf"	"$prefix/modeling/emft" 
"$prefix/modeling/mdt" 	"$prefix/modeling/m2t"	"$prefix/modeling/m2m"
"$prefix/modeling/gmf" 	"$prefix/tools/gef"
); 

oldAge=("14" "14" "21"); # ages, in days, as used below
egrepPattern="total|/home/data/httpd/download.eclipse.org/|/downloads/drops"; # used to add colour to output

# commandline flags
preview=0;
debug=0;
active=0;
function debug() # $3 can be set to "-n" 
{
	thresh=0;
	if [[ $2 -gt 0 ]]; then thresh=$2; fi
	if [[ $debug -gt $thresh ]]; then echo -e $3 "$1"; fi
} 

while [ "$#" -gt 0 ]; do
	case $1 in
		'-debug') debug=$2; debug "   $1 $2"; shift 1;;
		'-p'|'-preview') preview=1; debug "   $1"; shift 0;;
		'-a'|'-active') active=1; debug "   $1"; shift 0;;
		*) args=$args" $1"; debug "   $1"; shift 0;;
	esac
	shift 1
done

function execCmd()
{
	thresh=2;
	if [[ $preview -eq 1 ]] || [[ $debug -gt $thresh ]]; then # echo
		echo "$*" | perl -pe "s#^#    #";
	fi
	if [[ $preview -eq 0 ]]; then # run
		$*;
	fi
}

function purgeBuilds() # $age $dir $name-pattern
{
	a=$1; # age
	d=$2; # dir
	n=$3; # name

	# after x days, purge from $dir
	if [[ -d $d ]]; then 
		targets=$(find $d -type d -name "$n" -mtime +$a);
		if [[ $targets ]]; then
			debug "  Purge "$a" day old builds from $d";
			for f in $targets; do
				execCmd rm -fr $f;
			done
			debug " " 1;
		fi
	fi
}

function dushc()
{
	d=$*;
	du -shc $d | egrep $egrepPattern;
}

# get search directories	
i=0;
while [[ $i -lt ${#searchDirs[@]} ]]; do
	s=${searchDirs[$i]};
	if [[ -d $s ]]; then
		dirs=$dirs" "$(find $s -mindepth 2 -maxdepth 3 -name "drops"); # drops dir
	fi
	i=$((i+1));
done
	
if [[ $active -eq 1 ]]; then
	echo "[$(date +%H:%M:%S)] Started.";
	dushc $dirs;
	
	# how many dirs to clean?
	cnt=1;
	newDirs="";
	for dir in $dirs; do
		if [[ $(find $dir -maxdepth 1 -type d) != $dir ]]; then
			newDirs=$newDirs" "$dir; 
			cnt=$((cnt+1));
		fi
	done
	dirs=$newDirs;
	
	# start working 	
	i=0;
	for dir in $dirs; do
		if [[ $(find $dir -maxdepth 1 -type d) != $dir ]]; then 
			i=$((i+1));
			debug "[$i/$cnt] BEFORE: $(du -sh $dir)" 1 | egrep $egrepPattern;
			
			# 2. after x days, purge old [M] builds
			purgeBuilds ${oldAge[0]} $dir "M200*"  
			
			# 3. after x days, purge old [I] builds
			purgeBuilds ${oldAge[1]} $dir "I200*"  
			
			# 4. after x days, purge old [S] builds
			purgeBuilds ${oldAge[2]} $dir "S200*"  
			
			if [[ -d $oDir ]]; then
				debug "[$i/$cnt]  AFTER: $(du -shc $dir $oDir)" 1 | egrep $egrepPattern;
			else
				debug "[$i/$cnt]  AFTER: $(du -sh $dir)" 1 | egrep $egrepPattern;
			fi
		fi
	done
	echo "[$(date +%H:%M:%S)] Done.";
else
	echo -e "Nothing to do! Use ${red}-a$norm flag to activate this script.";
fi
dushc $dirs;
