#!/bin/bash

# run this as www-data or apache to update existing promoteToEclipse.*.properties files from CVS, or fetch new one(s) if missing
 
theDir=/home/www-data/build/modeling/scripts
cvsrootDefault=/cvsroot/modeling

if [[ $# -eq 0 ]]; then
	echo "Usage: $0 subprojectName [subprojectName2] [...]";
	echo "Example: $0 emf.ecoretools emf.search mdt.ocltools";
	echo " - or - "
	echo "$0 -a (to update all files in $theDir)";
	exit;
fi

targets="";

function getTargets()
{
	targets="";
	targetFiles=`cd $theDir; ls promoteToEclipse.*.properties`;
	for tf in $targetFiles; do
		sub=${tf%.properties}; sub=${sub#promoteToEclipse.};
		proj=`cat $tf | grep "projectName=" | grep -v "subprojectName=" | perl -pe "s/.*projectName=//"`;
		if [[ $proj == "emft" ]]; then proj="emf"; fi
		#echo "Got ${proj}.${sub}";
		targets=$targets" "${proj}"."${sub};
	done
}

if [[ $1 == "-a" ]]; then
	getTargets;
else
	while [ "$#" -gt 0 ]; do
		targets=$targets" "$1; shift 1;
	done
fi

tmpdir=`mktemp`; rm -fr $tmpdir; tmpdir=${tmpdir#/tmp/};
if [[ -d /tmp/$tmpdir ]]; then rm -fr /tmp/$tmpdir; fi
cd /tmp;

for t in $targets; do
	projectName=${t%.*};
	subprojectName=${t#*.};

	# hack for gef
	if [[ $projectName == "gef" ]]; then cvsroot="/cvsroot/tools"; else cvsroot=$cvsrootDefault; fi
	
	if [[ $projectName == "emf" ]] && [[ $subprojectName == "emf" ]]; then 
		moduleName=org.eclipse.emf/org.eclipse.emf.releng/scripts
	elif [[ $projectName == "emf" ]] && [[ $subprojectName != "emf" ]]; then 
		moduleName=org.eclipse.${projectName}/org.eclipse.emf.${subprojectName}.releng
	else
		moduleName=org.eclipse.${projectName}/org.eclipse.${subprojectName}.releng
	fi
	echo "Export $moduleName from HEAD ... "
	cvs -d :pserver:anonymous@dev.eclipse.org:${cvsroot} -Q export -r HEAD -d $tmpdir $moduleName
	if [[ ! -f /tmp/$tmpdir/promoteToEclipse.${subprojectName}.properties ]]; then
		echo "No promoteToEclipse.${subprojectName}.properties file found in CVS: nothing to do!";
	elif [[ -f /tmp/$tmpdir/promoteToEclipse.${subprojectName}.properties ]] && [[ ! -f $theDir/promoteToEclipse.${subprojectName}.properties ]]; then
		echo -n "Creating new file: $theDir/promoteToEclipse.${subprojectName}.properties ... ";
		cp -f /tmp/$tmpdir/promoteToEclipse.${subprojectName}.properties $theDir/
		chmod +w $theDir/promoteToEclipse.${subprojectName}.properties
		echo "done.";
	elif [[ -f /tmp/$tmpdir/promoteToEclipse.${subprojectName}.properties ]] && [[ `diff --brief /tmp/$tmpdir/promoteToEclipse.${subprojectName}.properties $theDir/promoteToEclipse.${subprojectName}.properties` ]]; then
		echo "Updating promoteToEclipse.${subprojectName}.properties ... ";
		diff -u $theDir/promoteToEclipse.${subprojectName}.properties /tmp/$tmpdir/promoteToEclipse.${subprojectName}.properties | perl -pe "s/^/   /"; 
		chmod +w $theDir/promoteToEclipse.${subprojectName}.properties
		cp -f /tmp/$tmpdir/promoteToEclipse.${subprojectName}.properties $theDir/
		chmod -w $theDir/promoteToEclipse.${subprojectName}.properties
		echo "done.";
	else
		echo "Nothing has changed in promoteToEclipse.${subprojectName}.properties";
	fi
	echo "";
	if [[ -d /tmp/$tmpdir ]]; then rm -fr /tmp/$tmpdir; fi
done
