#!/bin/bash

# script to check contents of lockfile dir and exec any if found

#Min Hr Mday Month Wday Cmd
# check every minute
#* * * * * /opt/public/cbi/build/common/scripts/lockfileWatcher.sh -writableBuildRoot /opt/public/modeling -all 2>&1 1>/dev/null

# if passed in proj, sub, version, filter directory contents for those; otherwise run on all files found
function usage()
{
	echo "usage: $0"
	echo "-proj    <OPTIONAL: name of the project to be built, eg. mdt>"
	echo "-sub     <OPTIONAL: shortname of the project to be built, eg. uml2tools>"
	echo "-version <OPTIONAL: version to use, eg., 1.0.0>"
	echo "-all     <OPTIONAL: if specifed, will run on all lockfiles found, instead of the above filters>"
	echo "-writableBuildRoot <OPTIONAL: dir where builds will occur, eg., /opt/public/cbi or /home/www-data>"
	exit;
}

if [[ $# -eq 0 ]]; then
	usage;
fi

writableBuildRoot="/opt/public/cbi"; # or /home/www-data

projectName="*";
subprojectName="*";
version="*";

# Create local variable based on the input
while [ "$#" -gt 0 ]; do
	case $1 in
		'-writableBuildRoot') 	writableBuildRoot=$2; 	shift 1;;
		'-proj') 				projectName=$2; 		shift 1;;
		'-sub')					subprojectName=$2; 		shift 1;;
		'-version')				version=$2; 			shift 1;;
		'-all')					projectName="*"; subprojectName="*"; version="*"; shift 0;;
 	esac
	shift 1;
done

lockfiledir=$writableBuildRoot"/tmp";
builddir=$writableBuildRoot"/build";
scriptsdir=$builddir"/common/scripts";

for f in $(find $lockfiledir -name "${projectName}-${subprojectName}_${version}.lock.txt"); do
  cmds=$(cat $f);
  vn=${cmds##*-version }; 	vn=${vn%% *}; 
  pn=${cmds##*-proj };    	pn=${pn%% *}; 
  sn=${cmds##*-sub }; 		sn=${sn%% *}; 
  ID=${cmds##*-buildType }; ID=${ID%% *}; id=${cmds##*-buildTimestamp }; id=${ID}${id%% *}; 
  mkdir -p ${builddir}/${pn}/${sn}/downloads/drops/${vn}/${id}/eclipse;
  chmod 775 ${scriptsdir}/*.sh
  ${scriptsdir}/start.sh $cmds >> ${builddir}/${pn}/${sn}/downloads/drops/${vn}/${id}/buildlog.txt 2>&1 &
done

# TODO port search cvs to common
for f in $(find $lockfiledir -name "*_updateSearchCVS.lock.txt"); do
  chmod 775 /opt/public/modeling/searchcvs/*.sh
  /opt/public/modeling/searchcvs/parsecvs_web.sh $(cat $f) 2>&1 1>/dev/null &
  rm -fr $f;
done
