#!/bin/bash

# TODO: rename $branch to $version, and $cvsbranch to $branch

# New order of steps in promote (others before or after)
#  0. uploads zips, jars, docs, etc.
#  1. publish RSS
#  2. run parsecvs.sh remotely from   http://build.eclipse.org/modeling/build/updateSearchCVS.php?projects[]=org.eclipse...&projects[]=org.eclipse...
#  3ai. block until done by checking  http://build.eclipse.org/modeling/emf/news/checkReleaseExists.php?project=emf&version={buildIDorAlias}      until value = 1
#  3aii. block until done by checking http://build.eclipse.org/modeling/mdt/news/checkReleaseExists.php?project=uml2&version={buildIDorAlias} until value = 1
#  3b. timeout if stuck for more than x mins
#  4. update bugs (RESOVED:FIXED -> VERIFIED:FIXED), announce in newsgroup, etc.

# script to update a given build, plus required files on \$downloadServerFullName from CVS
# run script on $buildServerFullName to push data to remote eclipse.org server
# Copyright \(c\) 2004-2008, IBM. Nick Boldt. codeslave\(at\)ca.ibm.com

# requires accompanying properties file, promoteToEclipse.properties, plus a few minimal commandline switches, like -sub ocl -branch 1.0.0 -buildID ...
# can also specify any other properties file with -f flag

# This script references the following other scripts:
# compareFolders (verify upload succeeded)
# fixJavadocs (javadocs)

###### ###### ###### ###### ###### ###### INSTRUCTIONS & OPTIONS ###### ###### ###### ###### ###### ###### 

norm="\033[0;39m";
grey="\033[1;30m";
green="\033[1;32m";
brown="\033[0;33m";
yellow="\033[1;33m";
blue="\033[1;34m";
cyan="\033[1;36m";
red="\033[1;31m";

function basichelp ()
{
	echo -e "You can either specify a CVS path to your properties file and its branch/tag with $red-c$norm, or use $red-sub$norm to specify a file in the current dir.";
	echo -e "For details on $red-sub$norm and all other flags & overrides, use $brown-h$norm or $brown-help$norm.";
	
	echo "";
	echo -e "usage:   $blue${0##*/}$norm $yellow-d \$CVSROOT$norm $red-c \$projectNamespace/\$subprojectReleng/promoteToEclipse.\$sub.properties,branchOrTag$norm";
	echo "" 
	echo -e "example: $blue${0##*/}$norm $red-c org.eclipse.emf/org.eclipse.emf.query.releng/promoteToEclipse.query.properties,R1_1_maintenance$norm"
	echo -e "example: $blue${0##*/}$norm $yellow-d :pserver:anonymous@dev.eclipse.org:/cvsroot/tools$norm \\"
	echo -e "                               $red-c org.eclipse.gef/org.eclipse.gef.releng/promoteToEclipse.gef.properties,HEAD$norm"
	echo "";
}

function morehelp ()
{
	echo -e "If not using a properties file in CVS, specify the local file used by this script BEFORE any overrides like $red-branch$norm or $red-buildID$norm."
	echo "";
	echo -e "usage:   $blue${0##*/}$norm";
	echo -e "$red-sub$norm          <REQUIRED: MUST BE FIRST FLAG; if ./promoteToEclipse.${red}sub$norm.properties not found, try ./promoteToEclipse.properties>"
	echo -e "$red-branch$norm       <REQUIRED: release version of the files to be promoted, eg., 1.0.0, 1.0.1 (overrides property file)>"
	echo -e "$red-buildID$norm      <REQUIRED: ID of the build>"
	echo "";
	echo -e "$red-cvsbranch$norm    <REQUIRED for RSS feeds: cvs branch of the release, eg., R2_2_maintenance, HEAD>"
	echo -e "$brown-branchIES$norm    <REQUIRED for IES map file: eg., R3_2_maintenance, HEAD>"
	echo ""
	echo -e "$yellow-q$norm, $yellow-Q$norm                     <OPTIONAL: scp, unzip, cvs checkout: quiet or VERY QUIET output>"
	echo -e "$yellow-coordsite$norm, $yellow-coordsiteonly$norm <OPTIONAL: update coordinated site, eg. $grey-coordsite ganymede$norm or $grey-coordsiteonly ganymede$norm>";
	echo -e "$yellow-announce$norm, $yellow-announceonly$norm   <OPTIONAL: post announcement in newsgroup - see properties file for settings>"
	echo -e "  $yellow-announcedelay$norm           <OPTIONAL: if used, delay announcement by specified number of seconds>"
	echo -e "$yellow-addSDK$norm                    <OPTIONAL: if used, add the SDK zip to the specified dependencies file for use with other builds>"
	echo -e "$yellow-email$norm                     <OPTIONAL: email to notify when done>"
	echo ""
	echo -e "-f            <OPTIONAL: if used, MUST BE FIRST FLAG; use specific file instead of ./promoteToEclipse.${red}sub$norm.properties>"
	echo "-user         <username on *.eclipse.org (default is $USER (\$USER))>"
	echo "-debug        <default 0; can be increased to 1 or 2>"
	echo "-tmpfolder    <folder to use when working> (optional)"
	echo "-noclean      <don't clean up when done (leave tmpfolder)>"

	# no/only options (9): drop, jars, rss, IES, docs, searchcvs, bugz
	echo ""
	echo "-nodrop, -droponly <do NOT / ONLY upload drop>"
	echo "  -noCompareDropsFolders <after uploading the drop, DO NOT compare source and target for matching MD5s, etc.>"
	echo "-nojars, -jarsonly <do NOT / ONLY do Update Manager jars create & upload step>";
	echo "  -basebuilderBranch  <org.eclipse.releng.basebuilder CVS branch, such as M2_32 or M3_32>"
	echo "  -noCompareUMFolders <after uploading the UM jars, DO NOT compare source and target for matching MD5s, etc.>"
	echo "  -notrackstats       <do not track stats in site*.xml>"
	echo "  -refreshSiteXMLOnly <refresh site*.xml from CVS and update on remote>"
	
	echo ""
	echo "-norss,  -rssonly  <do NOT / ONLY gen RSS feed file update for the given buildID & branch> (optional)"
	echo -e "  $red-cvsbranch$norm       <REQUIRED for RSS feeds: cvs branch of the release, eg., R2_2_maintenance, HEAD>"
	echo "  -feedURL         <override value set in feedPublish.*.properties, eg. for use with N builds>"
	echo "  -feedFile        <override value set in feedPublish.*.properties, eg. for use with N builds>"
	echo ""

	echo "-noIES,  -IESonly  <do NOT / ONLY gen IES map file update for the given buildID & branch> (optional)"
	echo -e "  $brown-branchIES$norm       <REQUIRED for IES map file: eg., R3_2_maintenance, HEAD>"
	echo "  -userIES         <override value set in .properties file, eg., -userIES nboldt>"
	echo "  -buildAlias      <REQUIRED if doing an -IESonly promote for a build which is not located on this server,"
	echo "                    in order to set \$buildAlias because it cannot be calculated from the SDK zip's name>"         
	echo ""
	echo "-nodocs, -docsonly           <do NOT / ONLY do javadocs step>";
	echo "-nosearchcvs, -searchcvsonly <do NOT / ONLY do Search CVS database update>";

	echo ""
	echo -e "$grey-nobugz$norm, $grey-bugzonly$norm           <do NOT / ONLY close RESOLVED:FIXED bugs> (optional; see bugs 171745, 191571, 207383)";

	echo "";
	echo -e "$grey-nomaven$norm, $grey-mavenonly$norm         <do NOT / ONLY gen Maven jars> (optional)";
}

function exampleshelp ()
{
	echo ""
	echo -e "To promote a drop, you need at least $red-sub$norm, $red-branch$norm, $red-buildID$norm, $red-cvsbranch$norm, and $brown-branchIES$norm (if applicable), with optional $yellow-announce$norm and $yellow-Q$norm:"
	echo ""
	echo -e "  $blue${0##*/} $red-sub ocl  $yellow-Q -announce $red-branch 1.0.2 $red-buildID M200612071439 $red-cvsbranch R1_0_maintenance $brown-branchIES R3_2_maintenance$norm"
	echo -e "  $blue${0##*/} $red-sub uml2 $yellow-Q -announce $red-branch 2.1.0 $red-buildID I200510101000 $red-cvsbranch HEAD $brown-branchIES HEAD$norm"
	echo ""
	echo -e "  $blue${0##*/} $red-sub emf  $yellow-Q -coordsiteonly callisto $red-branch 2.2.2 -buildID M200702010000$green 2>&1 | tee ~/promo_\`date +%Y%m%d_%H%M%S\`.txt$norm"
	echo -e "  $blue${0##*/} $red-sub emf  $yellow-Q -announceonly $red-branch 2.3.0 -buildID I200611030200$green 2>&1 | tee ~/promo_\`date +%Y%m%d_%H%M%S\`.txt$norm"
	echo ""
}

if [ $# -lt 1 ]; then
	basichelp;
	exit 1;
elif [[ $1 = "-h" ]] || [[ $1 = "--help" ]]; then
	basichelp;
	morehelp;
	exampleshelp;
	exit 1;
fi

###### ###### ###### ###### ###### ###### BEGIN SETUP ###### ###### ###### ###### ###### ###### 

# default to default properties file
defaultPropertiesFile=./promoteToEclipse.properties
propertiesFile=""; # value set by -c
propertiesFiles=""; # set of values to iterate 
propertiesFileCmd=""; # contains either "" or "-f /path/to/somefile.properties"
projRelengRoot=":pserver:anonymous@dev.eclipse.org:/cvsroot/modeling"; # default if not specified when building
buildAlias="";

bugz=0;
quietSSH="-v";
depsFile="";
announcedelay=0;
hostname="`hostname -f`";
relengCommonScriptsDir=$(dirname $0)

builduser=$(id www-data 2>&1 | grep "No such user" -c);
if [[ $builduser -eq 0 ]]; then
	builduser="www-data";
else 
	builduser=$(id apache 2>&1 | grep "No such user" -c);
	if [[ $builduser -eq 0 ]]; then
		builduser="apache";
	else
		builduser="$USER";
	fi
fi

# get path to PHP interpreter
PHP=php
if [ -x /usr/bin/php ]; then
	PHP=/usr/bin/php
elif [ -x /usr/bin/php4 ]; then
	PHP=/usr/bin/php4
elif [ -x /usr/bin/php5 ]; then
	PHP=/usr/bin/php5
else
	PHP=php
fi

getBuildCommonFile ()
{
	webRoot=/var/www/html; if [[ ! -d $webRoot ]]; then webRoot=/opt/public/modeling/public_html/; fi
	dlsPath=${localWebDir#$webRoot/}; # /var/www/html/modeling/$projectName or /var/www/html/tools/$projectName --> tools/gef or modeling/emf  
	webPath=${localWebDir#$webRoot/}; # /var/www/html/modeling/$projectName or /var/www/html/tools/$projectName --> tools/gef or modeling/emf  
	buildCommonFile=$webRoot/$webPath/$subprojectName/build/_common.php;
	if [[ -f $buildCommonFile ]]; then
		parent="modeling";
	else
		WARN="[promote] [1] can't find $buildCommonFile.";
		webPath=$projectName;
	 	buildCommonFile=$webRoot/$projectName/build/_common.php;
	 	if [[ -f $buildCommonFile ]]; then
			parent="NONE";
		else
			echo $WARN;	echo "[promote] [2] can't find $buildCommonFile: must abort."; 
			sendFailureEmail PROMOTE:FAILED:7; exit 2;
		fi
	fi
}

getDefaultEmail ()
{
	defaultEmail="";
	if [[ ! $buildCommonFile ]]; then getBuildCommonFile; fi
	emailDefault=$(cat $buildCommonFile | grep "EmailDefault"); 
	emailDefault=${emailDefault#*=> \"}; 
	emailDefault=${emailDefault%%\"*};
	emailDefault=${emailDefault//,/};
	if [[ $debug -gt 1 ]]; then echo "[email] emailDefault = $emailDefault"; fi
	if [[ ! $email ]]; then 
		emailList=$emailDefault;
	elif [[ $email ]] && [[ $emailDefault ]]; then
		emailList=$email" "$emailDefault;
	fi
	if [[ $debug -gt 2 ]]; then echo "[email] emailList = $emailList"; fi
	# prevent dupe entries
	allEm="";
	for em in $emailList; do
		if [[ ${allEm##*$em*} == $allEm ]]; then # email is new 
			if [[ $defaultEmail ]]; then
				defaultEmail=$defaultEmail",";
			fi 
			defaultEmail=$defaultEmail$em;
		else
			if [[ $debug -gt 2 ]]; then echo "[email] $em already added."; fi
		fi 
		allEm=$allEm$em;
	done
	if [[ $debug -gt 2 ]]; then echo "[email] defaultEmail = $defaultEmail"; fi
}

sendFailureEmail () 
{
	getDefaultEmail;
	if [[ $defaultEmail ]]; then
		cmd="$PHP -q $relengCommonScriptsDir/sendEmail.php -email $defaultEmail -hostname $hostname"
		if [[ $projectName ]]; then cmd=$cmd" -proj $projectName"; fi
		if [[ $subprojectName ]]; then cmd=$cmd" -sub $subprojectName"; fi
		if [[ $cvsbranch ]]; then cmd=$cmd" -branch $cvsbranch"; fi
		if [[ $branch ]]; then cmd=$cmd" -version $branch"; fi
		if [[ $buildID ]]; then cmd=$cmd" -buildID $buildID"; fi
		if [[ $parent ]]; then cmd=$cmd" -parent $parent"; fi
		cmd=$cmd" -promote true -buildStatus $1";
		$relengCommonScriptsDir/executeCommand.sh "$cmd";
	fi
}

loaded=0;
function loadCVSPropertiesFile()
{
	# Input: org.eclipse.emf/org.eclipse.emf.query.releng/promoteToEclipse.query.properties,R1_1_maintenance
	tmpProperties=`mktemp -d`; tmpProperties=${tmpProperties#/tmp/};
	in=$1;
	if [[ ${in##*,*} ]]; then in=$in,HEAD; fi # default branch is HEAD
	cvsPropertiesPath=$in; cvsPropertiesPath=${cvsPropertiesPath%,*}; # org.eclipse.emf/org.eclipse.emf.query.releng/promoteToEclipse.query.properties
	cvsPropertiesBranch=$in; cvsPropertiesBranch=${cvsPropertiesBranch#*,}; # R1_1_maintenance
	cvsPropertiesFile=${cvsPropertiesPath##*/}; # promoteToEclipse.query.properties
	subprojectName=${cvsPropertiesPath##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # query
	#echo "cvsPropertiesPath=$cvsPropertiesPath"; echo "cvsPropertiesBranch=$cvsPropertiesBranch"; echo "cvsPropertiesFile=$cvsPropertiesFile"; echo "subprojectName=$subprojectName";
	pushd /tmp >/dev/null;
	cvs -d $projRelengRoot -Q ex -r $cvsPropertiesBranch -d $tmpProperties $cvsPropertiesPath
	if [ -r /tmp/$tmpProperties/$cvsPropertiesFile ]; then 
		subprojectName=${cvsPropertiesFile##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # set a default value in case not set in the properties file 
		echo "   -c $in [loading $cvsPropertiesFile from $cvsPropertiesBranch, sub = $subprojectName ... "; 
		. /tmp/$tmpProperties/$cvsPropertiesFile;
		propertiesFile=/tmp/$tmpProperties/$cvsPropertiesFile
		propertiesFileCmd="-f $propertiesFile";
		loaded=1;
		echo " done]";
	else 
		echo "[promote] Properties file $cvsPropertiesPath not found in CVS for branch/tag $cvsPropertiesBranch. Exiting..."; 
		rm -fr /tmp/$tmpProperties; 
		sendFailureEmail PROMOTE:FAILED:1; exit 2; 
	fi; 
	
	if [[ ! $subprojectName ]]; then # if not set in properties file 
		subprojectName=${cvsPropertiesPath##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # query
	fi
	popd >/dev/null;
}

# for use with -*only flags, zero out all variables
# no/only options (8): drop, jars, rss, IES, docs, bugz, searchcvs, (maven)
# yes/only options(2): (coordsite), announce
function allZero()
{
   dodrop=0;
   UMjars=0;
      RSS=0;
      IES=0;
   dodocs=0;
searchCVS=0;
     bugz=0;
    maven=0;
depsFile="";
}

# Create local variable based on the input
echo " "
echo "[promote] Started `date +%H:%M:%S`. Executing with the following options:"
while [ "$#" -gt 0 ]; do
	case $1 in
		'-d'|'-projRelengRoot') projRelengRoot=$2; echo "   $1 $2"; shift 1;;
		'-parent') parent=$2; echo "   $1 $2"; shift 1;;
		'-c')    loadCVSPropertiesFile $2; shift 1;;
		'-f')    propertiesFile=$2;
			if [[ -r $propertiesFile ]]; then
				subprojectName=${propertiesFile##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # set a default value in case not set in the properties file 
				echo "   $1 $2 [loading, sub = $subprojectName ... "; 
				. $propertiesFile; 
				propertiesFileCmd="-f $propertiesFile";
				loaded=1;
				echo "done]";
			else 
				echo "[promote] Properties file $propertiesFile not found. Exiting...";
				sendFailureEmail PROMOTE:FAILED:2; exit 2; 
			fi; 
			shift 1;;
		'-sub')  subprojectName=$2;
			# chain them together in order of priority: subproj specific one, default
			propertiesFiles=$propertiesFile" $relengCommonScriptsDir/promoteToEclipse."$subprojectName".properties "$defaultPropertiesFile; 
			if [[ $loaded -eq 0 ]]; then 
				for propFile in $propertiesFiles; do 
					if [[ $loaded -eq 0 ]] && [[ -r $propFile ]]; then 
						echo -n "   $1 $2 [loading $propFile ... ";
						. $propFile; 
						propertiesFileCmd="-f $propFile";
						echo "done]"; 
						loaded=1; 
					fi; 
				done
			fi
			if [[ $loaded -eq 0 ]]; then
				echo "    [Can't load any of: $propertiesFiles. Exiting!]";
				sendFailureEmail PROMOTE:FAILED:3; exit 2;
			fi
			shift 1;;

	# build details

		'-branch')   branch=$2; echo "   $1 $2"; shift 1;;
		'-buildID') buildID=$2; echo "   $1 $2"; shift 1;;

	# user/email

		'-user')     user=$2;   echo "   $1 $2"; shift 1;;
		'-email')   	 email=$2; echo "   $1 $2"; shift 1;;

	# no/only options (8): drop, jars, rss, IES, docs, searchcvs, bugz, maven
	
		'-droponly') allZero; dodrop=1; echo "   $1"; shift 0;;
		'-nodrop') 			  dodrop=0; echo "   $1"; shift 0;;
		'-noCompareDropsFolders') noCompareDropsFolders=1; echo "   $1"; shift 0;;

		'-jarsonly') allZero;  UMjars=1; echo "   $1"; shift 0;;
		'-nojars')             UMjars=0; echo "   $1"; shift 0;;
		'-basebuilderBranch') basebuilderBranch="-basebuilderBranch $2"; echo "   $1 $2"; shift 1;;
		'-noCompareUMFolders') noCompareUMFolders="-noCompareUMFolders"; echo "   $1";    shift 0;;
		'-notrackstats')       notrackstats="-notrackstats";			 echo "   $1";    shift 0;;
		'-refreshSiteXMLOnly') allZero; refreshSiteXMLOnly="true";       echo "   $1";    shift 0;;

		'-rssonly') allZero; RSS=1;	echo "   $1"; shift 0;;
		'-norss') 			 RSS=0; echo "   $1"; shift 0;;
		'-cvsbranch') cvsbranch=$2;	echo "   $1 $2"; shift 1;;
		'-feedURL')	  feedURL="-DfeedURL=$2"; echo "   $1 $2"; shift 1;;
		'-feedFile')  feedFile="-Dfile=$2";   echo "   $1 $2"; shift 1;;

		'-IESonly') allZero; IES=1;	echo "   $1"; shift 0;;
		'-noIES') 			 IES=0; echo "   $1"; shift 0;;
		'-branchIES') branchIES=$2;	echo "   $1 $2"; shift 1;;
		'-userIES') userIES=$2; 	echo "   $1 $2"; shift 1;;
		'-buildAlias') buildAlias=$2;	echo "   $1 $2"; shift 1;;

		'-docsonly') allZero; dodocs=1; echo "   $1"; shift 0;;
		'-nodocs') 			  dodocs=0; echo "   $1"; shift 0;; 

		'-searchcvsonly') allZero; searchCVS=1; echo "   $1"; shift 0;;
		'-nosearchcvs') 		   searchCVS=0; echo "   $1"; shift 0;; 

		'-bugzonly') allZero; bugz=1; echo "   $1"; shift 0;;
		'-nobugz') 			  bugz=0; echo "   $1"; shift 0;;

		'-mavenonly') allZero; maven=1;	echo "   $1"; shift 0;;
		'-nomaven')  		   maven=0; echo "   $1"; shift 0;;

	# coordination options
	
		'-coordsite') 			   coordsite=1; coordsiteName=$2; echo "   $1 $2"; shift 1;;
		'-coordsiteonly') allZero; coordsite=1; coordsiteName=$2; echo "   $1 $2"; shift 1;;
		'-addSDK') 				   				   depsFile="$2"; echo "   $1 $2"; shift 1;; # [207007]
		'-addSDKonly')    allZero; 				   depsFile="$2"; echo "   $1 $2"; shift 1;; 

	# announcement options
	
		'-announce') 			   announce=1;  echo "   $1"; shift 0;;
		'-announceonly') allZero;  announce=1;  echo "   $1"; shift 0;;
		'-announcedelay')  announcedelay="$2";  echo "   $1 $2"; shift 1;; # [206706]

	# debug options
	
		'-tmpfolder') tempfold=$2; echo "   $1 $2";	shift 1;;
		'-q') quietCVS=-q; quiet=-q; quietSSH=""; echo "   $1"; shift 0;;
		'-Q') quietCVS=-Q; quiet=-q; quietSSH="-q"; echo "   $1"; shift 0;;
		'-debug') debug=$2; echo "   $1 $2"; shift 1;;
		'-noclean') noclean=1; echo "   $1"; shift 0;;

	esac
	shift 1
done

if [[ ! $subprojectName ]] && [[ $propertiesFileCmd ]]; then # if not set in properties file 
	subprojectName=${propertiesFileCmd##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; 
fi

if [ "$subprojectName" = "" ]; then # no value set!
  echo "[promote] No subproject name set in properties file or by -sub flag. Script cannot continue. Exiting...";
  sendFailureEmail PROMOTE:FAILED:4; exit 2;
fi

if [ "$branch" = "" ]; then # no value set!
  echo "[promote] No branch value set in properties file or by -branch flag. Script cannot continue. Exiting...";
  sendFailureEmail PROMOTE:FAILED:5; exit 2;
fi

if [ "$buildID" = "" ]; then # no value set!
  echo "[promote] No build ID value set in properties file or by -buildID flag. Script cannot continue. Exiting...";
  sendFailureEmail PROMOTE:FAILED:6; exit 2;
fi

# create default temp folder
mkdir -p /home/$user/tmp;

###### ###### ###### ###### ###### VARIABLES DERIVED FROM $user ###### ###### ###### ###### ###### 

# temp folder base (make 'em unique so concurrent builds don't overlap)
tempfold=/home/$user/tmp/promoteToEclipse-$projectName-$subprojectName-$user-`date +%Y%m%d_%H%M%S`

#users (for ssh and cvs connections)
buildServerCVSUser=$user"@"$buildServerFullName
eclipseCVSUser=$user"@"$eclipseServerFullName
eclipseSSHUser=$user"@"$downloadServerFullName
eclipseSSHUserHome=$(ssh $eclipseSSHUser "echo \$HOME");

#cvs paths
coordsiteCVSRep=:ext:$eclipseCVSUser:/cvsroot/callisto

# if no value for userIES, default to commandline -user or value in properties file
if [ "$userIES" = "" ]; then userIES=$user; fi
IESCVSRep=":ext:"$userIES"@ottcvs1.ottawa.ibm.com:/home/cvs/com.ibm.ies.releng"

###### ###### ###### ###### ###### SETUP DONE, DEFINE METHODS ###### ###### ###### ###### ###### 

getBuildIDactual ()
{
	if [[ $buildAlias ]]; then
		buildIDactual=$buildAlias
	else
		buildIDactual=`find $buildDropsDir/$branch/$buildID -iname "$SDKfilenamepattern"`
		buildIDactual=${buildIDactual##*-}; # trim up to -
		buildIDactual=${buildIDactual%%\.zip}; # trim off .zip
		#echo $buildIDactual
	fi
}

if [[ ! $buildCommonFile ]]; then getBuildCommonFile; fi

hasProblem="";
hasFailed="";
# should set noclean=1, hasProblem/hasFailed & cc: default email address
# if there's a problem in the build 
checkBuildStatus ()
{
	projectNameActual=$projectName;
	checkBuildStatusURL="http://$hostname/modeling/build/checkBuildStatus.php?parent=$parent&top=$projectName&project=$subprojectName&version=$version";
	checkBuildStatusURL=$checkBuildStatusURL"&buildID=$buildID";
	result=$(wget "$checkBuildStatusURL" -O - 2>/dev/null);
	hasProblem=$(echo $result | egrep "FAIL|ERROR| F| E"); 
	hasFailed=$(echo $result | egrep "FAIL| F"); 
	isUnknown=$(echo $result | egrep "UNKNOWN");
	if [[ $hasFailed ]]; then noclean=1; fi
	
	# special case for emf/emft
	if [[ $isUnknown ]] && [[ $projectName == "emf" ]]; then 
		projectName2="emft";
		checkBuildStatusURL="http://$hostname/modeling/build/checkBuildStatus.php?parent=$parent&top=$projectName2&project=$subprojectName&version=$version";
		checkBuildStatusURL=$checkBuildStatusURL"&buildID=$buildID";
		result=$(wget "$checkBuildStatusURL" -O - 2>/dev/null);
		hasProblem=$(echo $result | egrep "FAIL|ERROR| F| E"); 
		hasFailed=$(echo $result | egrep "FAIL| F"); 
		isUnknown=$(echo $result | egrep "UNKNOWN"); 
		if [[ ! $isUnknown ]]; then
			projectNameActual=$projectName2;
		fi
		if [[ $hasFailed ]]; then noclean=1; fi
	fi
	if [[ $hasProblem ]]; then
		getDefaultEmail;
	fi
}	

# must call checkBuildStatus before sendEmail to ensure we have all the right email addresses listed (user-defined + default/backup)
sendEmail () 
{
	if [[ $email ]]; then
		$relengCommonScriptsDir/executeCommand.sh "$PHP -q $relengCommonScriptsDir/sendEmail.php \
		  -email $email -proj $projectNameActual -sub $subprojectName \
		  -branch $cvsbranch -version $branch -buildID $buildID \
		  -hostname $hostname -parent $parent \
		  -promote true";
	fi
}

###### ###### ###### ###### METHODS DONE, BEGIN WORK HERE ###### ###### ###### ###### 

buildIDactual=buildID;
webuser=$(grep www /etc/passwd | head -1 | perl -pe "s/(^[^:]+):.+$/\1/g");

if [ $dodrop -eq 0 ]; then
	echo "[promote] Upload new drop - omitted."
else
	echo "[promote] Remove any temp files left over from a -noclean build"
	for f in \
		`find $buildDropsDir/$branch/$buildID -type d -name "org.eclipse*releng*"` \
		$buildDropsDir/$branch/$buildID/testing $buildDropsDir/$branch/$buildID/eclipse; do
		if [ -d $f ]; then sudo -u $webuser rm -fr $f; fi
	done		
	echo "[promote] [`date +%H:%M:%S`] Create target drop folder $projectDropsDir/$branch/$buildID on remote box started:"
	ssh $eclipseSSHUser mkdir -p $projectDropsDir/$branch/$buildID
	echo "[promote] [`date +%H:%M:%S`] Create target drop folder $projectDropsDir/$branch/$buildID on remote box done."

	echo "[promote] [`date +%H:%M:%S`] SCP build folder $branch/$buildID onto remote box started:" 
	rsync -ar $quietSSH --rsh=ssh --exclude='*-Update-*.zip*' --exclude='*-Master-*.zip*' $buildDropsDir/$branch/$buildID $eclipseSSHUser:$projectDropsDir/$branch/
	echo "[promote] [`date +%H:%M:%S`] SCP build folder $branch/$buildID onto remote box done."

	if [ $noCompareDropsFolders -eq 0 ]; then
		### CHECK MD5s and compare dir filesizes for match (du -s)
		echo "[promote] [`date +%H:%M:%S`] Comparing local and remote folders to ensure SCP completeness... "
		$relengCommonScriptsDir/compareFolders.sh -user $user -local $buildDropsDir/$branch/$buildID -remote $projectDropsDir/$branch/$buildID -server $eclipseSSHUser
		returnCode=$?
		if [ $returnCode -gt 0 ]; then
			echo "[promote] [`date +%H:%M:%S`] ERROR! Script exiting with code $returnCode from compareFolders.sh"
			exit $returnCode;
		fi
	else
		echo "[promote] [`date +%H:%M:%S`] Comparing local and remote folders to ensure SCP completeness ... omitted."
	fi

	# removed 20070608
	#echo "[promote] [`date +%H:%M:%S`] CHMOD build folder $branch/$buildID on remote box to give group perms started:"
	#	ssh $eclipseSSHUser "
	#		chmod -R $eclipsePermsDir $projectDropsDir/$branch/$buildID
	#		chgrp -fR $eclipseUserGroup $projectDropsDir/$branch/$buildID
	#	"
	#	## must keep closing quote on preceeding line to close the ssh command section
	#echo "[promote] [`date +%H:%M:%S`] CHMOD build folder $branch/$buildID on remote box to give group perms done."
fi

# add build to dependencies file?
if [[ $depsFile != "" ]]; then
	if [[ -f $depsFile ]]; then
		sudo -u $builduser chmod g+w $depsFile; sudo -u $builduser chgrp www $depsFile;
		depNum=$(cat $depsFile | grep "$subprojectName=" | tail -1); depNum=${depNum%%$subprojectName=*};
		SDKURL=$(find $buildDropsDir/$branch/$buildID -maxdepth 1 -name "$SDKfilenamepattern" | tail -1);
		if [[ $SDKURL != "" ]]; then 
			SDKURL=${SDKURL##$buildDropsDir/$branch/$buildID}; SDKURL=${SDKURL##*/}; 
			SDKURL=http://$hostname/$dlsPath/$subprojectName/downloads/drops/$branch/$buildID/$SDKURL; # TODO fix for GEF
			if [[ -w $depsFile ]]; then 
				echo "$depNum$subprojectName=$SDKURL" >> $depsFile;
				echo "[promote] $SDKURL ($depNum$subprojectName) appended to $depsFile.";
			else
				echo "[promote] *** WARNING: File $depsFile is not writable. Add this manually:";
				echo "$depNum$subprojectName=$SDKURL"
				echo "[promote] ***";
			fi
		else
			echo "[promote] *** WARNING: no SDK zip found in $buildDropsDir/$branch/$buildID. ***"; 
		fi
	else
		echo "[promote] *** WARNING: cannot store SDK. File $depsFile does not exist. ***";
	fi
fi

function refreshUpdates ()
{
	# purge updates site*.xml and refresh anonymously from cvs
	pushd $localWebDir/updates >/dev/null;
	for f in $(find $localWebDir/updates -type f -name "site*.xml" -o -name ".#site*"); do   
		rm -f $f;
	done
	echo ":pserver:anonymous@dev.eclipse.org:/cvsroot/org.eclipse" > $localWebDir/updates/CVS/Root
	cvs -Q up -Pd >/dev/null 2>&1;
	chmod -R g+w site*.xml;
	chgrp -R $buildUserGroup site*.xml;
	popd >/dev/null;
}

if [[ $refreshSiteXMLOnly ]]; then
	echo "[promote] Refresh updates folder (site*.xml) ...";
	refreshUpdates;
	echo "[promote] Promote site*.xml to $downloadServerFullName ...";
	pushd $localWebDir/updates >/dev/null;
	scp -r $quiet site*.xml $user"@"$downloadServerFullName:$projectWWWDir/updates;
	popd >/dev/null;
else
	if [[ $UMjars -eq 0 ]]; then
		echo "[promote] Create & promote Update Manager jars to download ... omitted [-nojars]."
	else
		echo "[promote] [`date +%H:%M:%S`] Create & promote Update Manager jars to download started:"
	
		echo "[promote] Refresh updates folder (site*.xml) ...";
		refreshUpdates;
			
		echo "[promote] Running buildUpdateSite.sh:"
		cmd="./buildUpdateSite.sh $propertiesFileCmd -sub $subprojectName -user $user $quietCVS -branch $branch -buildID $buildID -catgXMLOnly -promote $notrackstats $basebuilderBranch -debug $debug";
		if [[ $parent ]]; then cmd=$cmd" -parent $parent"; fi 
		if [[ $coordsiteName ]]; then cmd=$cmd" -coordsite $coordsiteName"; fi
		pushd $relengCommonScriptsDir >/dev/null; $cmd; popd >/dev/null;
		echo "[promote] [`date +%H:%M:%S`] Create & promote Update Manager jars to download done."
	fi
fi

function refreshFeeds ()
{
	# purge feeds folder and refresh from cvs	
	cd $localWebDir;
	rm -fr $localWebDir/feeds;
	cvs $quiet up -Pd feeds;
	chmod -R g+w feeds;
	chgrp -R $buildUserGroup feeds;
}

### Update Eclipse Project RSS feed file
if [ "$RSS" -eq 0 ]; then
	echo "[promote] Update RSS feed - omitted."
elif [ "$RSS" -eq 1 ]; then
	echo "[promote] Update RSS feed ..."

	echo "[promote] Refresh RSS feeds folder..."
	refreshFeeds;
	
	cd $buildScriptsDir;

	if [ "$cvsbranch" = "" ]; then # no value set!
	  echo "[promote] No CVS branch value set in properties file or by -cvsbranch flag. Script cannot continue. Exiting...";
	  sendFailureEmail PROMOTE:FAILED:8; exit 2;
	fi
		
	getBuildIDactual; buildAlias=$buildIDactual;
	buildType=${buildID:0:1};
	typeSuffix=""; if [ "$buildType" = "N" ]; then typeSuffix="\-N"; fi
	if [[ $debug -gt 0 ]]; then echo "[promote] Using buildAlias = $buildIDactual, branch = $branch, buildID = $buildID"; fi

	# find dependencyURLs from $buildDropsDir/$branch/$buildID/buildlog.txt
	# http://download.eclipse.org/downloads/drops/M20060609-1217/eclipse-SDK-M20060609-1217-linux-gtk.tar.gz
	# http://download.eclipse.org/modeling/emf/emf/downloads/drops/2.2.0/I200606150000/emf-sdo-xsd-SDK-I200606150000.zip
	dependencyURLs="";
	for dep in `head -50 $buildDropsDir/$branch/$buildID/buildlog.txt | grep "\-URL" | sed -e "s/^[^hf]\\+//g" -e "s/fullmoon\..\\+\.ibm\.com/download.eclipse.org/g" | sort | uniq`; do 
		if [ "x$dependencyURLs" != "x" ]; then dependencyURLs="$dependencyURLs,"; fi # join with commas
		dependencyURLs=$dependencyURLs"$dep";
		if [[ $debug -gt 0 ]]; then echo "[promote] Using URL = $dep"; fi			
	done
	# http://download.eclipse.org/downloads/drops/M20060609-1217/eclipse-SDK-M20060609-1217-linux-gtk.tar.gz,http://download.eclipse.org/modeling/emf/emf/downloads/drops/2.2.0/I200606150000/emf-sdo-xsd-SDK-I200606150000.zip
	
	export JAVA_HOME=/opt/sun-java2-5.0;
	export ANT_HOME=/opt/apache-ant-1.7.1;

	# extract jar from CVS if not available (first time only - this will only work if run as web user (www-data, apache) on emf.toro or emft.eclipse 
	#+ not a problem on build.eclipse running as modelingBuild
	if [ ! -f feedTools.jar ]; then $ANT_HOME/bin/ant -f feedWatchSetup.xml; fi
	
	mkdir -p $tempfold/rss; 
	# replace %%project%% (ocl, transaction, etc.); %%typeSuffix%% ("-N" or ""); %%user%% (cdamus, nickb, etc.) with actual values
	SDKzip=$(find $buildDropsDir/$branch/$buildID -name "$SDKfilenamepattern"); SDKzip=${SDKzip##*/}; # trim path
	if [[ ! $SDKzip ]]; then
	  echo "No SDK zip found for $buildDropsDir/$branch/$buildID/$SDKfilenamepattern -- script cannot continue. Exiting... "; 
	  sendFailureEmail PROMOTE:FAILED:9; exit 2;
	fi
	echo "[promote] Using SDK = $SDKzip"; 
	cat feedPublish.$projectName.properties | sed \
		-e "s/%%user%%/$user/g" \
		-e "s/%%typeSuffix%%/$typeSuffix/g" \
		-e "s/%%project%%/$subprojectName/g" \
		-e "s/%%SDKzip%%/$SDKzip/g" \
		> $tempfold/rss/feedPublish.$subprojectName.properties
	
	CLASSPATH="$JAVA_HOME/lib/rt.jar:"$ANT_HOME/lib/ant.jar":"$ANT_HOME/lib/ant-launcher.jar;
	cmd="$JAVA_HOME/bin/java -Dant.home=$ANT_HOME -Dant.library.dir=$JAVA_HOME/lib -classpath $CLASSPATH org.apache.tools.ant.launch.Launcher"
	cmd=$cmd" -buildfile feedPublish.xml -propertyfile $tempfold/rss/feedPublish.$subprojectName.properties"
	
	# override values in properties file with current values (or assign dynamic/missing values)
	# TODO: rename $branch to $version, and $cvsbranch to $branch
	cmd=$cmd" -Dproject=$subprojectName -Dversion=$branch -Dbranch=$cvsbranch -DbuildID=$buildID -DbuildAlias=$buildAlias -DbuildType=$buildType";
	cmd=$cmd" -DdependencyURLs=$dependencyURLs -Ddebug=$debug $feedURL $feedFile";
	echo ""; echo $cmd | perl -pe "s/ -/\n  -/g" | perl -pe "s/\.jar:/\.jar\n    :/g"; echo "";
	$cmd;
	
	# note: created files will not have the correct group ownership / perms
	echo "[promote] feed update done."
fi

if [ "$IES" -eq 0 ]; then
	echo "[promote] Update IES map file - omitted."
elif [ "$IES" -eq 1 ]; then
	# update the IES mapping file: checkout file, edit it, then check back in (commit)
	if [ "$branchIES" = "" ]; then # no value set!
  		echo "[promote] No IES branch value set in properties file or by -branchIES flag. Script cannot continue. Exiting...";
  		sendFailureEmail PROMOTE:FAILED:10; exit 2;
	fi

	getBuildIDactual;
	# create file in /home/cvs/com.ibm.ies.releng/com.ibm.ies.releng/maps/
	if [[ $projectName == $subprojectName ]] || [[ ! $subprojectName ]]; then
		IESmapfile=$projectName".map"; # EMF, GEF
	else
		IESmapfile=$projectName"-"$subprojectName".map"; # all others
	fi
	echo "[promote] Got actual buildID: $buildIDactual, IES branch: $branchIES, mapfile: com.ibm.ies.releng/maps/$IESmapfile";

	# setup
	tmpfolder=$tempfold/ies; mkdir -p $tmpfolder/1; cd $tmpfolder;

	# checkout
	if [ "$branchIES" != "HEAD" ]; then branchIES="-r "$branchIES; else branchIES=""; fi
	cvscmd="cvs -d $IESCVSRep $quietCVS co $branchIES -P -d 1 com.ibm.ies.releng/maps";
	if [[ $debug -gt 0 ]]; then echo "[promote] [`date +%H:%M:%S`] "$cvscmd; fi
	$cvscmd;
	echo "[promote] [`date +%H:%M:%S`] done."
	if [[ $projectName == $subprojectName ]] && [[ $subprojectName == "gef" ]]; then
		IESwebPath=http://fullmoon.ottawa.ibm.com/${dlsPath}/downloads/drops/$branch/$buildID/; # GEF
	else
		IESwebPath=http://fullmoon.ottawa.ibm.com/${dlsPath}/$subprojectName/downloads/drops/$branch/$buildID/; # all in Modeling
	fi
	#edit file, then make a copy in the other folder
	echo "[promote] Writing $IESmapfile ...";
	outfile=$tmpfolder/1/$IESmapfile

	index=0;
	element_count=${#IESmapfileArray[@]}
	while [ "$index"  -lt "$element_count" ]; do
	  # eg., echo "$projectName-SDK-$buildIDactual.zip=$IESwebPath     | | | sdk | $projectName"    >> $outfile.tmp ;
	  txt=${IESmapfileArray[$index]/buildIDactual/$buildIDactual}; txt=${txt/webPath/$IESwebPath};
	  echo $txt >> $outfile.tmp;
	  let "index = $index + 1";
	done

	## adds optional cvs add command
	cd $tmpfolder/1 ;
	if [ ! -f $IESmapfile ]; then
		echo "[promote] [`date +%H:%M:%S`] add $IESmapfile";	## add if not found
		mv -f $IESmapfile.tmp $IESmapfile ;
		cvs -d $IESCVSRep $quietCVS add -k v -m "promoteToEclipse: new map file" $IESmapfile;
	else
		dif=`diff -q $IESmapfile.tmp $IESmapfile`; # echo $dif;
		if [ "x$dif" = "x" ]; then # remove temp file, it's the same as the previous
			echo "[promote] [`date +%H:%M:%S`] $IESmapfile exists, no change ";	## no add if found
			rm -f $IESmapfile.tmp ;
		else # files differ
			echo "[promote] [`date +%H:%M:%S`] $IESmapfile exists, updating ";	## no add if found
			mv -f $IESmapfile.tmp $IESmapfile ;
		fi
	fi

	#check in
	cvs -d $IESCVSRep $quietCVS ci -m "promoteToEclipse: $branch/$buildID";
	echo "[promote] [`date +%H:%M:%S`] done." ;
	
fi

if [ "$dodocs" -eq 0 ]; then
	echo "[promote] Unzip new javadocs - omitted."
else

	echo "[promote] [`date +%H:%M:%S`] Javadoc creation started: "
	index=0;
	element_count=${#javadocTargetArray[@]}
	while [ "$index"  -lt "$element_count" ]; do
	  pushd $relengCommonScriptsDir >/dev/null; ./fixJavadocs.sh $propertiesFileCmd ${javadocTargetArray[$index]} $quiet -user $user -branch $branch -buildID $buildID; popd >/dev/null;
	  let "index = $index + 1"
	done
	echo "[promote] [`date +%H:%M:%S`] Javadoc creation done."
fi

#  run parsecvs.sh remotely     http://build.eclipse.org/modeling/build/updateSearchCVS.php?projects[]=org.eclipse...&projects[]=org.eclipse...
#  block until done by checking http://build.eclipse.org/modeling/emf/news/checkReleaseExists.php?project=emf&version={buildIDorAlias}      until value = 1
#  block until done by checking http://build.eclipse.org/modeling/mdt/news/checkReleaseExists.php?project=uml2&version={buildIDorAlias} until value = 1
#  timeout if stuck for more than x mins

updateSearchCVSDisabled=1; # DISABLED by bug 250820
function updateSearchCVS ()
{
  if [[ $updateSearchCVSDisabled -ne 1 ]]; then 
	  echo "[promote] [`date +%H:%M:%S`] Update Search CVS & Release Notes database ...";
	  parsecvsCmd="http://build.eclipse.org/modeling/build/updateSearchCVS.php?";
	  if [[ $subprojectName == "emf" ]]; then
	  	i=0;
	  	while [[ $i -lt ${#cvsModulesArray[@]} ]]; do
	  		# projects[]=cvssrc/projectName-subprojectNamespace (cvssrc/emf-org.eclipse.emf.ecore.sdo) or
	  		# projects[]=cvssrc_branches/projectName-subprojectNamespace-cvsbranch (cvssrc_branches/mdt-org.eclipse.xsd-R2_2_maintenance)
	  		if [[ $cvsbranch = "HEAD" ]]; then
	    		parsecvsCmd=$parsecvsCmd"projects%5B%5D=cvssrc%2F"${cvsModulesArray[$i]}"&";
	    	else
				parsecvsCmd=$parsecvsCmd"projects%5B%5D=cvssrc_branches%2F"${cvsModulesArray[$i]}"-"$cvsbranch"&";
	    	fi
	    	let "i = $i + 1"
	  	done
	  elif [[ $projectName == "emf" ]] || [[ $projectName == "emft" ]]; then
		parsecvsCmd=$parsecvsCmd"projects%5B%5D=cvssrc%2F"$projectName"-org.eclipse.emf."$subprojectName"&";
	  else
	    parsecvsCmd=$parsecvsCmd"projects%5B%5D=cvssrc%2F"$projectName"-org.eclipse."$subprojectName"&";   
	  fi
	  tmpfile=`mktemp`.updateSearchCVS;
	  echo "[promote] wget -q -O $tmpfile \"$parsecvsCmd\"";
	  echo "";
	  wget -q -O $tmpfile $parsecvsCmd; sleep 2; grep "Error: no valid projects added" $tmpfile; grep "h3" -A2 $tmpfile; rm -f $tmpfile;
	  echo "";
	  echo "[promote] Parallel update job started.";
  else
  	  echo "[promote] [`date +%H:%M:%S`] Update Search CVS & Release Notes database: DISABLED on line 801 of promoteToEclipse.sh.";
  fi
} 

#if [[ $searchCVS -eq 1 ]]; then
#  updateSearchCVS; # DISABLED by bug 250820
#else
  echo "[promote] [`date +%H:%M:%S`] Update Release Notes & Search CVS database ... DISABLED.";
#fi
	
if [[ $bugz -eq 0 ]]; then
    echo "[promote] Change RESOLVED:FIXED bugs to VERIFIED:FIXED - omitted."
else
	getBuildIDactual;
	if [[ $updateSearchCVSDisabled -ne 1 ]]; then
		echo "[promote] [`date +%H:%M:%S`] Update Search CVS started: checking up to $searchCVSTimeout seconds for verification...";
		if [[ $checkReleaseExists -eq 1 ]]; then
			url="http://build.eclipse.org/$webPath/news/checkReleaseExists.php?project=$subprojectName&version=$buildIDactual";
			tmpfile=`mktemp`;
	  		wget -q -O $tmpfile "$url";
			# DISABLED by bug 250820
	  		#if [[ $(cat $tmpfile) -ne 1 ]]; then
	  			# force a database update
				#updateSearchCVS; 
	  		#fi
			rm -f $tmpfile;
	
			if [[ $searchCVSTimeout -gt 0 ]]; then
			  	elapsed=0;
				while [[ $elapsed -lt $searchCVSTimeout ]]; do
			    	echo -n "[promote] $elapsed: $url ... ";
			  		rm -f $tmpfile;
			  		wget -q -O $tmpfile "$url";
			  		if [[ $(cat $tmpfile) -ne 1 ]]; then echo ""; sleep 30s; (( elapsed += 30 ));
			  		else echo "done!"; break 1; fi
				done
				rm -f $tmpfile;
				if [[ $elapsed -ge $searchCVSTimeout ]]; then 
			  		echo "[promote] [`date +%H:%M:%S`] Update Search CVS may have taken more than $searchCVSTimeout seconds to complete.";
			  		echo "[promote] Giving up verification and continuing script. You may have to re-run the following step.";
			    fi
			fi
		fi
	else
  		echo "[promote] [`date +%H:%M:%S`] Update Search CVS & Release Notes database: DISABLED on line 801 of promoteToEclipse.sh.";
	fi
	echo "[promote] [`date +%H:%M:%S`] Changing RESOLVED:FIXED bugs to VERIFIED:FIXED ... ";
	
	pushd $relengCommonScriptsDir >/dev/null;
	
	# extract jar from CVS if not available (first time only) -- this will only work if run as www-data user
	if [ ! -f $relengCommonScriptsDir/bugTools.jar ]; then sudo -u www-data $ANT -f $relengCommonScriptsDir/updateBugStateSetup.xml; sudo -u www-data chmod +x $buildScriptsDir/*.sh; fi
		
	loginfile=/home/$user/.ssh/UpdateBugStateTask.properties
	bugzillaLoginID="";
	bugzillaLoginCookie="";
	
	getBuildIDactual;
	if [ "$buildIDactual" != "$buildID" ]; then
		buildAliasFlag="-buildAlias $buildIDactual";
		relnotesver=$buildIDactual;
	else
		buildAliasFlag="";
		relnotesver=$buildIDactual;
	fi
	
	relnotesURL="http://build.eclipse.org/$webPath/news/relnotes.php?project="$subprojectName"&version="$relnotesver"&bugzonly&strict";
	echo -n "[promote] Check $relnotesURL ... ";
	bugList=$(curl "$relnotesURL" 2>&1 | egrep "[0-9]+;[0-9]{0,6}"); echo -n " found "${bugList%;*}" bugs: "; bugList=${bugList#*;}; echo $bugList; 
		
	# EMF Special Case -- EMF + XSD
	if [ "$subprojectName" = "emf" ]; then 
		relnotesURL="http://build.eclipse.org/modeling/mdt/news/relnotes.php?project=xsd&version="$relnotesver"&bugzonly&strict";
		echo -n "[promote] Check $relnotesURL ... ";
		bugList2=$(curl "$relnotesURL" 2>&1 | egrep "[0-9]{5,6}"); echo -n " found "${bugList2%;*}" bugs: "; bugList2=${bugList2#*;}; echo $bugList2; 
		if [[ ! $bugList ]]; then bugList=$bugList2; else bugList=$bugList","$bugList2; fi
		bugList2="";
	fi
	
	echo "[promote] Found the following bugs for release $relnotesver: $bugList"; 
	
	if [ -r $loginfile ]; then
		bugzillaLoginID=`head -1 $loginfile`; bugzillaLoginID=${bugzillaLoginID#login=};
		bugzillaLoginCookie=`head -2 $loginfile | tail -1`; bugzillaLoginCookie=${bugzillaLoginCookie#logincookie=};
	fi
	
	cmd="$relengCommonScriptsDir/updateBugState.sh -login $bugzillaLoginID -logincookie $bugzillaLoginCookie -relengCommonScriptsDir $relengCommonScriptsDir \
	  -in RESOLVED:FIXED -out VERIFIED:FIXED -bugList $bugList $buildAliasFlag -buildID $buildID -branch $cvsbranch -version $branch";
	if [ $debug -gt 0 ]; then cmd=$cmd" -debug $debug"; fi
	
	if [ "x$bugzillaLoginID" == "x" ] || [ "x$bugzillaLoginCookie" == "x" ]; then
		if [ "x$bugzillaLoginID" == "x" ]; then       echo "[promote] ERROR: login cannot be read from "$loginfile".";
		elif [ "x$bugzillaLoginCookie" == "x" ]; then echo "[promote] ERROR: logincookie cannot be read from "$loginfile"."; fi
		echo "[promote] Run \`./bugzillaLogin.sh 2> $loginfile\` to regenerate login credentials."
		echo "[promote] Then use '-bugzonly' (or manually update bugs in Bugzilla).";		
	elif [ "x$bugList" == "x" ]; then 
		echo "[promote] Check the above query - no bugs found for this release. Nothing to do!";
	else 
		if [ $debug -gt 0 ]; then 
			echo -n "Running "; echo "$cmd" | perl -pe "s/ -/\n  -/g"; 
		else
			echo "[promote] Running updateBugState.sh"
		fi
		$cmd;
	fi
	echo "[promote] [`date +%H:%M:%S`] Bugzilla updated.";
	popd >/dev/null;
fi

if [[ $maven -eq 1 ]]; then
	if  [[ -f $buildScriptsDir/buildMaven.sh ]]; then
		echo "[promote] [`date +%H:%M:%S`] Create & promote Maven jars to download started:"
		echo "[promote] Running buildMaven.sh:"
		cd $buildScriptsDir; ./buildMaven.sh -sub "$subprojectName" -user $user -branch $branch -buildID $buildID -promote -debug $debug;
		echo "[promote] [`date +%H:%M:%S`] Create & promote Maven jars to download done."
	else
		echo "[promote] Maven jar creation is not yet supported.";
	fi
else
	echo "[promote] Create Maven jars ... omitted [-nomaven]."
fi

getBranchLabel () 
{
		if [[ "$buildIDactual" == "$branch" ]]; then						# echo "branch and actual are equal, need only $branch";
			branchLabel="$branch";
		elif [[ "${buildIDactual##$branch}" != "$buildIDactual" ]]; then	# echo "just need $buildIDactual";
			branchLabel="$buildIDactual";
		else														# echo "Need both $branch and $buildIDactual";
			branchLabel="$branch $buildIDactual";
		fi
}

if [[ $coordsite -eq 1 ]]; then
	if [[ ! $coordsiteBranch ]]; then coordsiteBranch="HEAD"; fi # set default if missing
	
	# setup
	tmpfolder=$tempfold/coordsite; mkdir -p $tmpfolder; cd $tempfold;
	qualifier=".v"${buildID:1};
	buildType=${buildID:0:1};
	case $buildType in 
		'N'|'M'|'I') buildTypeLabel="interim";;
		'S') buildTypeLabel="milestones";;
		'R') buildTypeLabel="releases";;
	esac
	if [[ $coordsiteName ]]; then coordsiteSuffix="-"$coordsiteName; else coordsiteSuffix=""; fi
	sitexmlPath="$buildTypeLabel/site"$coordsiteSuffix".xml"; # interim/site-ganymede.xml

	# TODO: add next coordsite format, if applicable
		
	featuresXMLArray=( "$projectName.sc" );
	cspecNameArray=( "org.eclipse.$projectName-sc" );
	featuresXMLfiltersArray=( "" );
	if [[ $subprojectName == "emf" ]]; then # special case -- generate two .sc files (TODO: split SDO out of EMF?)
		featuresXMLArray=( "emf-emf.sc" "mdt-xsd.sc");
		cspecNameArray=( "org.eclipse.emf-emf-sc" "org.eclipse.mdt-xsd-sc" );
		featuresXMLfiltersArray=( "emf" "xsd" );
	elif [[ $projectName == $subprojectName ]] && [[ $subprojectName != "emf" ]]; then # gmf
		true;
	elif [[ ! $subprojectName ]]; then # gef
		true;
	else # all other modeling projects
		featuresXMLArray=( "$projectName-$subprojectName.sc" );
		cspecNameArray=( "org.eclipse.$projectName-$subprojectName-sc" );
	fi

	echo "[coordsite] [`date +%H:%M:%S`] Update Coordinated Update Site";
	# checkout
	if [ "$coordsiteBranch" != "HEAD" ]; then coordsiteBranch="-r "$coordsiteBranch; else coordsiteBranch=""; fi
	# TODO: add next coordsite format, if applicable	
	cvscmd="cvs -d $coordsiteCVSRep -Q co $coordsiteBranch -d coordsite org.eclipse."$coordsiteName".sitecontributions";
	if [[ $debug -gt 0 ]]; then echo "[coordsite] [`date +%H:%M:%S`] "$cvscmd; fi
	$cvscmd;
	
	i=0;
	while [[ $i -lt ${#featuresXMLArray[@]} ]]; do
		featuresXML=${featuresXMLArray[$i]};
		cspecName=${cspecNameArray[$i]};
		featuresXMLfilters=${featuresXMLfiltersArray[$i]};
		(( i++ ));
		echo "[coordsite] [`date +%H:%M:%S`] [$i] $sitexmlPath -> $featuresXML ...";
		cd $localWebDir/updates/features;
		coordsiteZips=$(find $buildDropsDir/$branch/$buildID -name "$SDKfilenamepattern");
		features="";
		for coordsiteZip in $coordsiteZips; do
			echo "[coordsite] [`date +%H:%M:%S`] Searching for features in "${coordsiteZip#$buildDropsDir/}":";
			
			# get actual feature versions for the following defined list: org.eclipse.*sub_x.y.z.vqualifier.jar - by default, include runtime and sdk only
			# or override in promoteToEclipse.*.properties file
			if [[ ! $coordsiteFeaturePattern ]]; then coordsiteFeaturePattern=".*eclipse/features/org.eclipse..*(${subprojectName}|${subprojectName}.sdk)_.*\/$"; fi
			
			features=$features" "$(unzip -l $coordsiteZip | grep -v "META-INF" | egrep "${coordsiteFeaturePattern}" | perl -pe "s#.*eclipse/features/(.+)/#\1#g");
			if [[ $SDKfilenamepattern2 ]]; then # hack for qtv which requires two zips
				coordsiteZip=$(find $buildDropsDir/$branch/$buildID -name "$SDKfilenamepattern2");
				if [[ $coordsiteZip ]]; then  
					echo "[coordsite] [`date +%H:%M:%S`] Searching for features in "${coordsiteZip#$buildDropsDir/}":";
					features=$features" "$(unzip -l $coordsiteZip | grep -v "META-INF" | egrep "${coordsiteFeaturePattern}" | perl -pe "s#.*eclipse/features/(.+)/#\1#g");
				else
					echo "[coordsite] [`date +%H:%M:%S`] WARNING! No SDK zip found in $buildDropsDir/$branch/$buildID for pattern $SDKfilenamepattern2!";
				fi
			fi
		done
		
		cd $tmpfolder;
		if [[ ! -f $featuresXML ]]; then
		  touch $featuresXML; cvs -q add -k v -m "promoteToEclipse: new file" $featuresXML;
		else
		  echo -n '' > $featuresXML
		fi
	
		# TODO: add next coordsite format, if applicable	
		memberName=$projectNameVanity" Build Team";
		#memberEmail=${newsgroupPublisherEmail##*<}; memberEmail=${memberEmail%%>}; # echo $memberEmail;
		getDefaultEmail; 
		emailList=$(echo $defaultEmail | tr "," " "); #" "$memberEmail;
	
		echo '<?xml version="1.0" encoding="UTF-8" ?>
<sc:siteContribution
	xmlns="http://www.eclipse.org/buckminster/CSpec-1.0"
	xmlns:sc="http://www.eclipse.org/buckminster/SiteContribution-1.0"
	updateSite="${downloads}/'${dlsPath}'/updates/'$sitexmlPath'">' >> $featuresXML;
		echo -n "[coordsite] [`date +%H:%M:%S`] Set contact info: $memberName"; 
		for memberEmail in $emailList; do
			echo -n ", $memberEmail";
			echo '	<sc:member name="'$memberName'" email="'$memberEmail'" />' >> $featuresXML;
		done
		echo "";
		echo '	<sc:cspec name="'$cspecName'">
		<dependencies>' >> $featuresXML;
		featuresMain="";
		featuresEnable="";
		if [[ ! $coordsiteMainFeaturePattern ]]; then coordsiteMainFeaturePattern="org.eclipse..*(${subprojectName}|${subprojectName}.sdk)$"; fi
		for f in $features; do
			featureId=${f%%_*}; # trim from _ to end
			featureVersion=${f#*_}; # trim from start to _
			# collect main features vs. enabling features
			if [[ ! $featuresXMLfilters ]] || [[ $(echo $featureId | egrep -c "$featuresXMLfilters") -gt 0 ]]; then  
				if [[ $(echo $featureId | egrep -c "${coordsiteMainFeaturePattern}") -gt 0 ]]; then
					featuresMain=$featuresMain" "$featureId;
					echo "              :main: "$f; # to console only
				else
					featuresEnable=$featuresEnable" "$featureId;
					echo "            :enable: "$f; # to console only
				fi
				echo '			<dependency name="'$featureId'" versionDesignator="['$featureVersion']" />' >> $featuresXML;
			fi
		done
		echo '		</dependencies>
		<groups>' >> $featuresXML;

		if [[ $featuresEnable ]]; then
			echo '			<public name="Enabling Features">' >> $featuresXML;
			for f in $featuresEnable; do
				echo '				<attribute component="'$f'" />' >> $featuresXML;
			done
			echo '			</public>' >> $featuresXML;
		fi
		if [[ $featuresMain ]]; then
			if [[ $coordsiteMainFeatureName ]]; then
				echo '			<public name="'$coordsiteMainFeatureName'">' >> $featuresXML;
			else
				echo '			<public name="Models and Model Development">' >> $featuresXML; # default
			fi
			for f in $featuresMain; do
				echo '				<attribute component="'$f'" />' >> $featuresXML;
			done
			echo '			</public>' >> $featuresXML;
		fi
		echo '		</groups>
	</sc:cspec>
</sc:siteContribution>
' >> $featuresXML;
	
		#cat $featuresXML;		
	
		# commit changes
		cvs ci -m "promoteToEclipse: $sitexmlPath, $branch$qualifier" $featuresXML; 
		
	done
	echo "[coordsite] [`date +%H:%M:%S`] done.";
fi
	
if [[ $noclean -eq 0 ]]; then
	# cleanup temp space
	if [[ -d $tempfold/ ]]; then
		rm -fr $tempfold/;
		echo "[promote] Temporary files purged from $tempfold/."
	fi
	if [[ $tmpProperties ]]; then
		rm -fr /tmp/$tmpProperties
		echo "[promote] Temporary files purged from /tmp/$tmpProperties/"
	fi
else
	if [[ $tmpProperties ]]; then
		echo "[promote] Temporary files left in $tempfold/ and /tmp/$tmpProperties/. Please scrub them manually when done."
	else
		echo "[promote] Temporary files left in $tempfold/. Please scrub them manually when done."
	fi
fi

# send newsgroup notification
postfile=newsgroup-post.txt
if [[ $announce -eq 1 ]]; then
	echo -n "[announce] Announce new build in newsgroup";
	if [[ -r /home/$user/.ssh/$postfile ]]; then
		echo -n " ($newsgroup): ";
	  	elapsed=0; while [[ $elapsed -lt $announcedelay ]]; do echo -n "."; sleep 15s; (( elapsed += 15 )); done
		echo "";
		getBuildIDactual;
		getBranchLabel;
		mkdir -p $tempfold;
		head -2 /home/$user/.ssh/$postfile > $tempfold/$postfile;
		# this file should contain the following two lines:
		# authinfo user exquisitus
		# authinfo pass ********** (replace with real pwd)
		echo "post" >> $tempfold/$postfile;
		echo "From: $newsgroupPublisherEmail" >> $tempfold/$postfile;
		echo "Subject: [Announce] $projectNameVanity $branchLabel is available" >> $tempfold/$postfile;
		echo "Newsgroups: $newsgroup" >> $tempfold/$postfile;
		if [[ $newsgroupThreadReferences ]]; then # add support for threading under an existing post
			echo "References: $newsgroupThreadReferences" >> $tempfold/$postfile;
			echo "In-Reply-To: $newsgroupThreadReferences" >> $tempfold/$postfile;
		fi
		echo "" >> $tempfold/$postfile;
		echo "$releaseNotesURL&version=$buildIDactual" >> $tempfold/$postfile;
		echo $downloadsURL"&showAll=1&hlbuild="$buildID"#"$buildID >> $tempfold/$postfile;
		echo "" >> $tempfold/$postfile;
		echo "." >> $tempfold/$postfile;
		echo "quit" >> $tempfold/$postfile;
		cat $tempfold/$postfile | nc news.eclipse.org 119 2>&1 ;
		rm -fr $tempfold/$postfile;
		echo "[announce] done.";
	else
		echo ": $postfile not found or not readable, message delivery disabled."
	fi
fi

cd $relengCommonScriptsDir;
checkBuildStatus; sendEmail;

echo "[promote] Done `date +%H:%M:%S`"
echo ""

