#!/bin/bash

# this script must be run as www-data, which will in turn execute as the build's user via ssh

# 00 21 * * 2 sudo -u www-data /home/www-data/build/emf/scripts/promoteToEclipse_cron.sh \
#                -c org.eclipse.emf/org.eclipse.emf.releng/scripts/promoteToEclipse.emf.properties,HEAD -sub emf -branch 2.4.0 -type I \ 
#                -user nickb -announce -coordsite ganymede -email "nickboldt@gmail.com" > $HOME/cron_logs/promoteToEclipse_cron.sh.emf.24I.txt 2>&1
# 48 21 * * 3 sudo -u www-data /home/www-data/build/modeling/scripts/promoteToEclipse_cron.sh \
#                -c org.eclipse.emf/org.eclipse.emf.transaction.releng/promoteToEclipse.transaction.properties,HEAD -sub transaction -branch 1.2.0 -type I \
#                -user cdamus -announce -coordsite ganymede -email "cdamus@ca.ibm.com,nickboldt@gmail.com" > $HOME/cron_logs/promoteToEclipse_cron.sh.transaction.12I.txt 2>&1

#required
cvsProperties="";
subprojectName="";
branch="";
type="";
preview=0;
FORCE=0;

#optional or just passing thru
args="-Q";

if [ $# -eq 0 ]; then
  echo "[promo_cron] This is a wrapper for promoteToEclipse.sh to be run in a crontab.";
  echo "[promo_cron] BuildID will be calculated as the most recent folder in the drops dir which"
  echo "[promo_cron] starts with the appropriate prefix letter (N, I or M)."
  echo "";
  echo "[promo_cron] Must specify user, project, branch and type, and OPTIONALLY, feedURL & feedFile, eg:";
  echo "";
  echo "[promo_cron]   ./promoteToEclipse_cron.sh -c \$projectNamespace/\$subprojectNamespace/promoteToEclipse.\$subprojectName.properties,branchOrTag"
  echo "[promo_cron]     -user $USER -type M";
  echo "[promo_cron]         - or - ";
  echo "[promo_cron]   ./promoteToEclipse_cron.sh -user $USER -sub emf -branch 2.3.0 -type I";
  echo "[promo_cron]         - or - ";
  echo "[promo_cron]   ./promoteToEclipse_cron.sh -user $USER -sub emf -branch 2.3.0 -type N";
  echo "[promo_cron]     -feedURL http://download.eclipse.org/tools/emf/feeds/builds-emf-N.xml";
  echo "[promo_cron]     -feedFile /var/www/html/emf/feeds/builds-emf-N.xml";
  echo "";
  echo "[promo_cron] Note: if type = I or M, build will be promoted along with feed."
  echo "[promo_cron]       if type = N, only the feed will be promoted (-rssonly).";
  echo "";
  echo "additional flags: $0";
  echo "-p, -preview       <optional: don't actually run a promote, just echo the command for preview>"
  echo "-FORCE             <optional: don't check checkReleaseExists.php?project=\$subprojectName&version=\$buildID;" 
  echo "                              do a(nother) promote even if already done before>"
  exit 1;
fi

# Create local variable based on the input
echo " "
echo "[promo_cron] Started `date`. Executing with the following options:"
while [ "$#" -gt 0 ]; do
	case $1 in
		'-d'|'-projRelengRoot') projRelengRoot="$1 $2"; echo "   $1 $2"; shift 2;;
		'-parent') parent="$1 $2"; 						echo "   $1 $2"; shift 2;;
		'-c') cvsProperties=$2;						echo "   $1 $2"; shift 2;; 
		'-sub') subprojectName=$2;					echo "   $1 $2"; shift 2;;
		'-branch') branch=$2; args=$args" $1 $2"; 	echo "   $1 $2"; shift 2;;
		'-type') type=$2;      						echo "   $1 $2"; shift 2;;
		'-user') user=$2; args=$args" $1 $2"; 		echo "   $1 $2"; shift 2;;
		'-p'|'-preview') preview=1; 				echo "   $1";    shift 1;;
		'-FORCE') FORCE=1; 							echo "   $1";    shift 1;;
		*) args=$args" $1";    						echo "   $1";    shift 1;;
	esac
done

if [[ $type != "I" ]] && [[ $type != "M" ]]; then 
	echo "[promo_cron] Builds of type N, S, and R should not be promoted automatically by crontab!"; 
	exit 1;
fi

buildDir=$(cd /home/www-data/build/modeling/*/$subprojectName/ 2>&1; pwd); # support modeling
if [[ $(echo $buildDir | grep "No such file or directory" -c) -gt 0 ]]; then 
	buildDir=$(cd /home/www-data/build/$subprojectName/ 2>&1; pwd); # support GEF
	if [[ $(echo $buildDir | grep "No such file or directory" -c) -gt 0 ]]; then
		echo "Invalid subproject - can't enter /home/www-data/build/modeling/*/$subprojectName/ or /home/www-data/build/$subprojectName/ !"; 
		exit 1;
	fi
fi
#echo $buildDir; 

dropsDir=$(cd $buildDir/downloads/drops/$branch/ 2>&1; pwd);
if [[ $(echo $dropsDir | grep "No such file or directory" -c) -gt 0 ]]; then
 		echo "Invalid project or branch - can't enter /home/www-data/build/modeling/*/$subprojectName/downloads/drops/$branch or /home/www-data/build/$subprojectName/downloads/drops/$branch !"; 
	exit 1;
fi
#echo $dropsDir;

webPath=${buildDir##/home/www-data/build/}; webPath=${webPath%/*}; # echo $webPath; 
scriptsDir=/home/www-data/build/modeling/scripts/;
if [[ $subprojectName == "emf" ]]; then scriptsDir=/home/www-data/build/emf/scripts/; fi # EMF special case

if [ -d $scriptsDir ]; then cd $scriptsDir; else echo "[promo_cron] Invalid project - can't enter $scriptsDir!"; exit 1; fi
if [ ! -d $dropsDir ]; then echo "[promo_cron] Invalid project or branch - can't enter $dropsDir!"; exit 1; fi

buildID=$(cd $dropsDir && find . -maxdepth 1 -type d -name "$type*" -exec date -r {} +%s\ {} \; | sort | tail -1 | sed -e "s/[0-9]\+\ \.\///g");
if [[ ! $buildID ]]; then echo "[promo_cron] Error! No value found for \$buildID in $dropsDir: must abort."; exit 1; fi
	
if [[ $cvsProperties ]]; then cmd="$parent $projRelengRoot -c $cvsProperties"; else cmd="-sub $subprojectName"; fi

mkdir -p /home/www-data/promo_logs/;

if [[ $FORCE -eq 0 ]]; then
	# check to see if we've already promoted this build; if so, DON'T do it again
	url="http://build.eclipse.org/$webPath/news/checkReleaseExists.php?project=$subprojectName&version=$buildID";
	echo "[promo_cron] Checking $url"
	tmpfile=`mktemp`;
	wget -q -O $tmpfile "$url";
	if [[ $(cat $tmpfile) -eq 1 ]]; then
		echo "$subprojectName $branch $buildID already exists!";
		echo "[promo_cron] Use -FORCE flag to promote again. Script will now exit."
		echo "";
		rm -f $tmpfile;
		exit;
	fi
	rm -f $tmpfile;
fi
	
datestamp=$(date +%Y-%m-%d-%H.%M.%S);
log="/home/www-data/promo_logs/promo_log_${subprojectName}_${branch}.${buildID}_${datestamp}_cron.txt";
echo "[promo_cron] [$datestamp] Promoting $subprojectName $branch.$buildID as $user...";
if [[ $preview -eq 1 ]]; then 
	echo "";
	echo "/bin/bash -c \"exec /usr/bin/nohup /usr/bin/setsid ssh $user@$(hostname -f) \\\"cd /home/www-data/build/modeling/scripts; ./promoteToEclipse.sh \
	  $cmd -buildID $buildID $args \\\" >> $log 2>&1 &\"";
	echo "";
	echo "*** PREVIEW ONLY. No promote started. ***";
else
	/bin/bash -c "exec /usr/bin/nohup /usr/bin/setsid ssh $user@$(hostname -f) \"cd /home/www-data/build/modeling/scripts; ./promoteToEclipse.sh \
	  $cmd -buildID $buildID $args \" >> $log 2>&1 &"
	echo "[promo_cron] Log: $log";
fi
