#!/bin/bash
# this script simply wraps start.sh and puts a buildlog.txt in the build's folder
# required flags are -projectid and -version; the rest will be assumed or generated
# see start.sh for more on optional overrides for when assumed defaults do not work

relengCommonScriptsDir=$(cd $(dirname $0); pwd)

if [[ $# -eq 0 ]]; then
	$relengCommonScriptsDir/start.sh
	exit;
fi

args="";
writableBuildRoot=/opt/public/cbi/build
buildTimestamp=`date +%Y%m%d%H%M`
buildType=N

while [ "$#" -gt 0 ]; do
	case $1 in
		'-projectid')
			projectid=$2;
			# if x.y -> top.proj, proj=sub
			if [[ ${projectid%.*} == ${projectid%%.*} ]]; then # two-part projectid; single-match trim .* and greedy-match trim .* are the same
				topprojectName=${projectid%%.*}; # get first chunk
				projectName=${projectid##*.}; # get last chunk
				subprojectName=${projectName}; # proj == sub
			else # assume x.y.z -> top.proj.sub 
				topprojectName=${projectid%%.*}; # get first chunk
				subprojectName=${projectid##*.}; # get last chunk
				projectName=${projectid#${topprojectName}.}; # trim first chunk
				projectName=${projectName%.${subprojectName}}; # trim last chunk
			fi				
			#echo "Got: $topprojectName / $projectName / $subprojectName";
			args="$args $1 $2"; shift 1;;
		'-writableBuildRoot') 	writableBuildRoot=$2; 	args="$args $1 $2"; shift 1;;
		'-version')				version=$2; 			args="$args $1 $2"; shift 1;;
		'-buildType')			buildType=$2; 			args="$args $1 $2"; shift 1;;
		'-buildDir') 			buildDir=$2; 			args="$args $1 $2"; shift 1;;
		'-buildTimestamp') 		buildTimestamp=$2; 		args="$args $1 $2"; shift 1;;
		*) args="$args $1";;
	esac
	shift 1
done	

if [[ ! $buildDir ]]; then
	buildDir="${writableBuildRoot}/${projectid//.//}/downloads/drops/${version}/${buildType}${buildTimestamp}"
fi
mkdir -p $buildDir

logfile=$buildDir/buildlog.txt;
$relengCommonScriptsDir/start.sh $args 2>&1 >$logfile &
echo "Build started. See $logfile" 
