#!/bin/sh

login=0;
logincookie=0;
debug=1;
in="RESOLVED:FIXED";
out="VERIFIED:FIXED";
bugList="";
buildAlias="";
buildID="";
branch="";
version="";
relengCommonScriptsDir=$(dirname $0)

if [ $# -lt 1 ]; then
	echo "usage: updateBugState.sh"
	echo "  -login 1234         <you can get the value of login and logincookie by running org.eclipse.releng.basebuilder/";
	echo "  -logincookie 123456  /plugins/org.eclipse.build.tools/scripts_bugzilla/bugzillaLogin.sh>";
	echo "  -debug [0|1|2]      <defaults to 1 if not specified; 2 for more output, 0 for errors only>";
	echo "  -in      	       	<current state:resolution of bugs to change; default: RESOLVED:FIXED>";
	echo "  -out	     	    <target state:resolution of bugs; default: VERIFIED:FIXED>";
	echo "  -bugList            <comma-separated list of bugzilla numbers to process>";
	echo "  -buildAlias         <buildAlias  to use when commenting in bugzilla after making state change, eg., 2.0.4RC>";
	echo "  -buildID            <buildID to use when commenting in bugzilla after making state change, eg., S200606071234>";
	echo "  -branch             <branch to use when commenting in bugzilla after making state change, eg., R2_0_maintenance>";
	echo "  -version            <version to use when commenting in bugzilla after making state change, eg., 2.0.4>";
	echo "  -relengCommonScriptsDir   <path to bugTools.jar and updateBugStateSetup.xml; default '.'>";
	exit;
fi

echo "[updBug] Executing with the following options:"
while [ "$#" -gt 0 ]; do
	echo "   $1 $2";
	case $1 in
		'-login')		login=$2;;
		'-logincookie')	logincookie=$2;;
		'-debug')		debug=$2;;
		'-in')			in=$2;;
		'-out')			out=$2;;
		'-bugList')		bugList=$2;;
		'-buildAlias')	buildAlias=$2;;
		'-buildID')		buildID=$2;;
		'-branch')		branch=$2;;
		'-version')		version=$2;;
		'-relengCommonScriptsDir'|'-commonScriptsDir') relengCommonScriptsDir=$2;;
	esac
	shift 2
done

#################################################################

export JAVA_HOME=/opt/sun-java2-5.0;
export ANT_HOME=/opt/apache-ant-1.7.1;

# extract jar from CVS if not available (first time only)
if [ ! -f $relengCommonScriptsDir/bugTools.jar ]; then $ANT_HOME/bin/ant -f $relengCommonScriptsDir/updateBugStateSetup.xml; fi

#aasemble the command and its classpath
CLASSPATH="$JAVA_HOME/lib/rt.jar";
# note that com.sun.org.apache.xerces (Sun JDK 1.5, rt.jar) != org.apache.xerces (Ant 1.6.5, xercesImpl.jar) so must remove from classpath
#for f in `find $ANT_HOME/lib  -maxdepth 1 -name "*.jar" -type f -not -name "xercesImpl.jar"`; do CLASSPATH=$CLASSPATH":"$f; done
CLASSPATH=$CLASSPATH":"$ANT_HOME/lib/ant.jar":"$ANT_HOME/lib/ant-launcher.jar;
cmd="$JAVA_HOME/bin/java \
  -Dant.home=$ANT_HOME \
  -Dant.library.dir=$JAVA_HOME/lib \
  -classpath $CLASSPATH:$relengCommonScriptsDir/bugTools.jar \
  org.apache.tools.ant.launch.Launcher \
  -buildfile updateBugState.xml \
  	-Dlogin=$login -Dlogincookie=$logincookie -Ddebug=$debug \
  	-Din=$in -DbugList=$bugList -DbuildAlias=$buildAlias \
  	-Dout=$out -DbuildID=$buildID -Dbranch=$branch -Dversion=$version";

if [[ $debug -gt 0 ]]; then 
	echo -n "[updBug] Running "; echo $cmd | perl -pe "s/ -/\n  -/g" | perl -pe "s/:/\n    :/g"; echo "";
fi

# run the command
$cmd;
